#ifndef _ARCLIB_H
#define	_ARCLIB_H

#include "commdef.h"
#include "if.h"

#ifdef _WIN32
	#include <Winsock2.h>
#else
	#include <sys/time.h>
	#include <sys/times.h>
#endif

#include "sasbase.h"
#include "fibrebase.h"
#include "scsibase.h"

#pragma pack(1)

// flags of getting event
#define FLAG_ALL			0
#define FLAG_NEW			1

// Beeper settings
#define BEEPER_ON			1
#define BEEPER_OFF			0

// Host Mode
#define RAID				0
#define JBOD				1


#define DISABLE				0
#define ENABLE				1

#define	ON					0
#define OFF					1


#define	TYPE_64BIT_LBA			0
#define	TYPE_4K_FOR_WIN			1


// host type interface
// HostType 
#define HOST_PCI			1
#define HOST_PCIExpress		2
#define HOST_PATA			3
#define HOST_SATA1			4
#define HOST_SATA2			5
#define HOST_SCSI160		6
#define HOST_SCSI320		7
#define HOST_Fibre2G		8
#define HOST_Fibre4G		9
#define HOST_SAS			10

// TargetType
#define TARGET_PATA			1
#define TARGET_SATA			2
#define TARGET_SAS			3
#define TARGET_Fibre		4


// COM port attributes
typedef struct sCOM_ATTR
{
	BYTE comBaudRate;				// BaudRate: 0/1/2/3/4/5/6/7 (1200/2400/4800/9600/19200/38400/57600/115200)					
	BYTE comDataBits;				// DataBit: always 1 (8 bits)
	BYTE comStopBits;				// StopBit: (0:1, 1:2 stop bits)
	BYTE comParity;					// Parity: always 0 (no parity)
	BYTE comFlowControl;			// FlowControl: always 0 (no flow control)
} sCOM_ATTR, *pCOM_ATTR;

// Rebuild priority
enum _REBUILD_PRIORITY
{
	REBUILD_PRIORITY_ULOW,        // Ultra Low(5%)
	REBUILD_PRIORITY_LOW,         // Low(20%)
	REBUILD_PRIORITY_MEDIUM,      // Medium(50%)
	REBUILD_PRIORITY_HIGH,        // High(80%)
	TOTAL_REBUILD_PRIORITY
};

typedef struct sSYSTEM_INFO
{
	BYTE gsiVendorName[40];			// Vendor name
	BYTE gsiSerialNumber[16];		// Serial number
	BYTE gsiFirmVersion[16];		// Version of firmware 
	BYTE gsiBootVersion[16];		// Version of boot
	BYTE gsiMbVersion[16];			// Version of MBR(Master Boot Record)
	BYTE gsiModelName[8];			// Model name
	BYTE gsiLocalIp[4];				// Static IP if DHCP disabled
	BYTE gsiCurrentIp[4];			// Current IP Address, the value is either assigned by DHCP server if DHCP enabled or static IP if DHCP disabled.
	DWORD gsiTimeTick;				// Raid subsystem current time tick
	DWORD gsiCpuSpeed;				// CPU speed ( in MHz )
	DWORD gsiICache;				// CPU internal Instruction cache size ( in bytes )
	DWORD gsiDCache;				// CPU internal Data Cache size	( in bytes )
	DWORD gsiSCache;				// CPU secondary cache size	( in bytes )
	DWORD gsiMemorySize;			// Memory size ( in MBs )
	DWORD gsiMemorySpeed;			// Memory speed ( in MHz )
	DWORD gsiEvents;				// # of IDE events supported
	BYTE gsiMacAddress[6];			// MAC address if ethernet support
	BYTE gsiDhcp;					// 0: DHCP function disabled, 1: DHCP function enabled
	BYTE gsiBeeper;					// 0: Beeper disabled, 1: Beeper enabled
	BYTE gsiChannelUsage;			// 0: RAID , 1: JBOD
	BYTE gsiMaxAtaMode;				// Max ATA mode
	BYTE gsiSdramEcc; 				// 1: If ECC enabled
	BYTE gsiRebuildPriority;		// Initialize/rebuild/migration priority
	sCOM_ATTR gsiComA; 				// Attribute of COM1
	sCOM_ATTR gsiComB; 				// Attribute of COM2
	BYTE gsiIdeChannels;			// # of IDE channels supported
	BYTE gsiScsiHostChannels;		// # of SCSI host channels supported
	BYTE gsiIdeHostChannels;		// # of IDE host channels supported
	BYTE gsiMaxVolumeSet;			// # of VolumeSets supported
	BYTE gsiMaxRaidSet;				// # of RaidSet supported
	BYTE gsiEtherPort; 				// 1: If ethernet port supported
	BYTE gsiRaid6Engine; 			// 1: Raid6 engine supported
	BYTE gsiRs232Snmp;				// 1: SNMP through RS232
	BYTE gsiMIB_1;
	BYTE gsiMIB_2;
	DWORD gsiEnterpriseNum;			// Enterprise number of SNMP
    BYTE gsiHostType;               // Host type
    BYTE gsiTargetType;             // Target type
    BYTE gsiInBandEnable;           // 1: If Inband enabled
    BYTE gsiInBandChannel;          // The SCSI channel of inband device
    BYTE gsiInBandId;               // The SCSI ID of inband device
    BYTE gsiNumberEnclosures;		// # of Enclosures supported(SAS only)
	BYTE gsiR5060Supported;			// 1: Raid Level30/50/60 supported
	BYTE gsiNtpSupported;			// 1: NTP supported
	DWORD	gsiRes1;				// 
	u_int64_t	gsiSasAddress;		// controller's SAS address
	BYTE    gsiModelNameExt[8];     // 
    BYTE    gsiRes[60-4-8-8];       //
} sSYSTEM_INFO, *pSYSTEM_INFO;



//=======================================================
// Definitions of raidset status
//=======================================================
#define  RAID_NORMAL   		0x01	// Raid set in use
#define  RAID_IN_REBUILD	0x02    // Raid set is rebuilding
#define  RAID_COMPLETE  	0x04	// 0: If incomplete
#define  RAID_PASSTHROUGH 	0x80	// Pass through raid set


typedef struct sGUI_RAIDSET
{
	BYTE grsRaidSetName[16];	// 16 bytes raidset name
	DWORD grsCapacity;			// Minimum capacity in blocks required for each disk (member disk's minimum capacity)
	DWORD grsCapacityX;			// Reserved for 64 bit extension
	DWORD grsFailMask;			// Bit mask to indicate the failed drive (bit0 -- drive 0, bit1 -- drive 1 ....)
	BYTE grsDevArray[32];		// Array for member disk's channel#
	BYTE grsMemberDevices;		// Number of member disks in this raid set
	BYTE grsNewMemberDevices;	// New number of disks in this raid set (in case of raid EXPANDing)
	BYTE grsRaidState;			// Current state of RAID set
	BYTE grsVolumes;			// # of volumes created in this raid set
	BYTE grsVolumeList[16];		// Array of VOLUME NUMBER of the created volumes in this raid set
	BYTE grsRes1;				// Reserved
	BYTE grsRes2;				// Reserved
	BYTE grsFlag;				// Bit7: 1=>flag valid, Bit1/0: 00=>16 volumesets mode, 01=>128 volumesets mode
	BYTE grsFreeSegments;		// # of free segment in this raid set
	DWORD grsRawStripes[8];		// Free segment array, each segment is 0x200 blocks for each member disk
	DWORD grsRes4;				// Reserved
	DWORD grsRes5;				// Reserved
	DWORD grsRes6;				// Reserved
	BYTE grsVolumeListX[112];	// Array of VOLUME NUMBER of the created volumes in this raid set
	BYTE grsDevEncArray[32];	// Array for member enclosure#
} sGUI_RAIDSET, *pGUI_RAIDSET;





//=======================================================
// Definitions of volume status
// Array status == OR of all of its volumes
// gvsVolumeStatus
//=======================================================
#define  VOLUME_NORMAL				0x00
#define  VOLUME_INITIALIZING		0x01
#define  VOLUME_FAILED				0x02
#define  VOLUME_MIGRATING			0x04
#define  VOLUME_REBUILDING			0x08
#define  VOLUME_NEED_INIT			0x10
#define  VOLUME_NEED_MIGRATE		0x20
#define  VOLUME_INIT_FLAG			0x40
#define  VOLUME_NEED_REGEN			0x80 
#define  VOLUME_CHECKING			0x100
#define  VOLUME_NEED_CHECK			0x200
#define  VOLUME_MIGLBA_INVALID	    0x400
#define  VOLUME_MIGLBA_FAILED	    0x800
#define  VOLUME_CHECK_NO_SCRUB	    0x1000
#define  VOLUME_CHECK_NO_REGEN	    0x2000
#define	 VOLUME_NEED_CLONE          0x4000
#define	 VOLUME_CLONING             0x8000

//=======================================================
// Definitions of volume state
// gvsVolState
//=======================================================
#define	 VOLUME_R5060_VOLUME		0x000100	// The volume is a member of another volume which raidlevel 30/50/60
#define	 VOLUME_R5060_PARENT		0x100000	// The volume is a root volume of raidlevel 30/50/60
#define  VOLUME_WRITE_PROTECT       0x4000      // Write Protection
#define  VOLUME_ENCRYPTED           0x8000      // Encrypted volume
#define  VOLUME_KEY_VALIDATED       0x40000000  // Volume Unlock/Lock
//=======================================================
// Definitions of volume type
//=======================================================
#define  VOLUME_TYPE_STRIPING 		0  // Striping
#define  VOLUME_TYPE_MIRROR  		1  // Mirror 
#define  VOLUME_TYPE_RAID3  		2  // RAID 3 volume
#define  VOLUME_TYPE_RAID5  		3  // RAID 5
#define  VOLUME_TYPE_RAID6  		4  // RAID 6
#define  VOLUME_TYPE_PASSTHROUGH 	5  // Pass through volume

//=======================================================
// Definitions of volume stripe size
//=======================================================
#define STRIPE_SIZE_4K		0
#define STRIPE_SIZE_8K		1
#define STRIPE_SIZE_16K		2
#define STRIPE_SIZE_32K		3
#define STRIPE_SIZE_64K		4
#define STRIPE_SIZE_128K	5


//=======================================================
// Definitions of encryption key type
//=======================================================
#define VOLUME_ENCRYPT_PASSWORD256	0
#define VOLUME_ENCRYPT_AES256 		1
#define VOLUME_ENCRYPT_PASSWORD128	2
#define VOLUME_ENCRYPT_AES128 		3

//=======================================================
// Definitions of encryption key assigning
//=======================================================
#define VOLUME_ENCRYPT_KEY_FOUND	    1

enum _SCSI_SPEED_MODE
{
	SCSI_SPEED_ASYNC = 0,
	SCSI_SPEED_20,
	SCSI_SPEED_40,
	SCSI_SPEED_80,
	SCSI_SPEED_160,
	TOTAL_SCSI_SPEED
};

enum _IDE_SPEED_MODE
{
	IDE_SPEED_33 = 0,
	IDE_SPEED_66,
	IDE_SPEED_100,
	IDE_SPEED_133,
	TOTAL_IDE_SPEED
};


enum _SATA_SPEED_MODE
{
	SATA_SPEED_150 = 0,
	SATA_SPEED_150NCQ,
	SATA_SPEED_300,
	SATA_SPEED_300NCQ,
	TOTAL_SATA_SPEED
};


#define BACKGROUND_INIT		0
#define FOREGROUND_INIT		1

enum _UDMA_MODE
{
	UDMA_MODE_0 = 0, // Ultra DMA Mode 0 235ns 16.7 MB/s ATA-4BUltra ATA/16
	UDMA_MODE_1, // Ultra DMA Mode 1 160ns 25 MB/s ATA-4BUltra ATA/25
	UDMA_MODE_2, // Ultra DMA Mode 2 120ns 33.3 MB/s ATA-4BUltra ATA/33 
	UDMA_MODE_3, // Ultra DMA Mode 3 90ns 44.4 MB/s ATA-5BUltra ATA/44
	UDMA_MODE_4, // Ultra DMA Mode 4 60ns 66.6 MB/s ATA-5BUltra ATA/66 
	UDMA_MODE_5, // Ultra DMA Mode 5 40ns 100 MB/s ATA-6BUltra ATA/100 
	UDMA_MODE_6, // Ultra DMA Mode 6 30ns 133MB/s ATA-7BUltra ATA/133
	TOTAL_UDMA_MODE
};

typedef struct sGUI_VOLUMESET
{
	BYTE gvsVolumeName[16];		// 16 bytes volumeset name
	DWORD gvsCapacity;			// Volume Capacity (in blocks)
	DWORD gvsCapacityX;			// Reserved for 64 bits
	DWORD gvsFailMask;			// Failed mask for the volume (same as raidset)
	DWORD gvsStripeSize;		// (value in blocks) 8--4K, 16--8K, 32--16K, 64--32K, 128--64K, 256--128K
	DWORD gvsNewFailMask;		// For internal use only
	DWORD gvsNewStripeSize;		// New stripe size ( used for MIGRATION)
	DWORD gvsVolumeStatus;		// See above ( volume status )
	DWORD gvsProgress; 			// Progress for the volume if in (initializing/rebuilding/migrating)
	sSCSI_ATTR gvsScsi; 		// SCSI attribute
	BYTE gvsMemberDisks;		// # of member disks
	BYTE gvsRaidLevel; 			// Raid Level of this volume
	BYTE gvsNewMemberDisks;		// Used for migration
	BYTE gvsNewRaidLevel;		// Used for migration
	BYTE gvsRaidSetNumber;		// Raid set number that the volumeset belongs to
	BYTE gvsVolState0;			// Reserved
	DWORD gvsHostSpeed;			// Reserved
	DWORD gvsVolState;			// See above( volume state )
	BYTE gvsVolArray[16];		// Child-volume table
	BYTE gvsNum5060Volumes;		// Number of volumes in the field 'gvsVolArray'
	BYTE gvsAesKeyType;         // See above ( encryption key type )
    BYTE gvsAesKeyAssigned;     // 1:assigned otherwise:not assigned
    BYTE gvsRes;
    DWORD gvsChanMask;          // Channel mask for dual controller
    BYTE gvsSerial[16];
    BYTE gvsRes1[12];
} sGUI_VOLUMESET, *pGUI_VOLUMESET;

//=======================================================
// Definitions of device(disk) status
//=======================================================
#define  DEV_INITIALIZED 	0x80  // Device initialized
#define  DEV_AVAILABLE 		0x40  // Device available for use
#define  DEV_HOT_SPARE 		0x20  // Device is reserved for HOT SPARE
#define  DEV_FAILED  		0x10  // Device failed
#define  DEV_ATTACHED 		0x08  // Device attached
#define  DEV_OFFLINE		0x04  // Device offlined
#define	 DEV_PASSTHROUGH	0x02  // Pass through device

//=======================================================
// Definitions of device(disk) status for SAS controller
//=======================================================
#define  DEV_SATA_DEVICE	0x100 // Device is a SATA type
#define	 DEV_POWER_DOWN	    0x200 // Device power down


enum _TYPE_HOT_SPARE
{
	HOT_SPARE_GLOBAL = 0,			// global hot spare (default)
	HOT_SPARE_LOCAL_RAID,			// local to specific raidset
	HOT_SPARE_LOCAL_ENC,			// local to enclosure
	TOTAL_TYPE_HOT_SPARE		
};

typedef struct sGUI_PHY_DRV
{
	BYTE	gpdModelName[40];		// Model name
	BYTE	gpdSerialNumber[20];	// Serial number
	BYTE	gpdFirmRev[8];			// Firmware version
	DWORD	gpdCapacity;			// Capacity in blocks ( A block = 512 bytes )
	DWORD	gpdCapacityX;			// Reserved for expansion
	BYTE	gpdDeviceState;			// See above, for SAS controller: this field is treated as the low-byte of device state
	BYTE	gpdPioMode;				// For SAS controller: this field is treated as the high-byte of device state	                                
	BYTE	gpdCurrentUdmaMode;		// Current UDMA mode
	BYTE	gpdUdmaMode;			// Max supported UDMA mode
	BYTE	gpdHotSpareType;		// 0: Global 1: Dedicated for RaidSet 2: Dedicated for Enclosure
	BYTE	gpdRaidNumber;			// 0xff if not belongs to a raid set
	sSCSI_ATTR	gpdScsi;			// SCSI attributes
	LONG	gpdMediaErrorCount;		// Media error count
	LONG	gpdTimeOutCount;		// Timeout count
	u_int64_t	gpdSasAddress;		// SAS only
	union
	{
		struct
		{
			sSMART_DATA gpdSmart[TOTAL_SMART_ITEMS];		// see sasbase.h 'enum _SMART_ITEMS'
		} sata;	// SMART data for SATA disk
		struct
		{
			WORD		gpdRotationRPM;						// The rotation(RPM) of disk
			WORD		gpdTemperature;						// The temperature of disk
			WORD		gpdSmartSenseCode;					// The SMART sense code
			WORD		gpdReserved;
			u_int64_t   gpdAttribute[TOTAL_SAS_ATTR_ITEMS];	// The SAS disk attributes, see sasbase.h '_SAS_ATTRIBUTE_ITEMS'
		} sas;	// Attributes for SAS disk
	} u;
	WORD	gpdHotSpareRaidNumber;	// RaidSet# if gpdHotSpareType == 1
} sGUI_PHY_DRV, *pGUI_PHY_DRV;


//=======================================================
// Definition for event buffer
//=======================================================
// Event category
enum
{
	EVENT_NO_EVENT,
	EVENT_RAIDSET,
	EVENT_VOLUMESET,
	EVENT_DEVICE,
	EVENT_HOST,
	EVENT_HW_MONITOR,
	EVENT_NEW_83782D,	
};
// Event type
// 1. RaidSet event 
enum
{
	RS_EVT_CREATE,
	RS_EVT_DELETE,
	RS_EVT_EXPAND,
	RS_EVT_REBUILD,
	RS_EVT_DEGRADED,
	RS_EVT_NO_EVENT,	
};
// 2. volumeset event
enum
{
	VS_EVT_INITIALIZING,
	VS_EVT_REBUILDING,
	VS_EVT_MIGRATING,
	VS_EVT_CHECKING,
	VS_EVT_COMPLETE_INIT,
	VS_EVT_COMPLETE_REBUILD,
	VS_EVT_COMPLETE_MIGRATING,
	VS_EVT_COMPLETE_CHECKING,
	VS_EVT_CREATE,
	VS_EVT_DELETE,
	VS_EVT_MODIFY,
	VS_EVT_DEGRADED,
	VS_EVT_FAILED,
	VS_EVT_REVIVED,

	// init/rebuild/migration ABORT & STOP event
	VS_EVT_ABORT_INIT,
	VS_EVT_ABORT_REBUILD,
	VS_EVT_ABORT_MIGRATING,
	VS_EVT_ABORT_CHECKING,
	VS_EVT_STOP_INIT,
	VS_EVT_STOP_REBUILD,
	VS_EVT_STOP_MIGRATING,
	VS_EVT_STOP_CHECKING,
	VS_EVT_UNSUPPORTED_STRIPE_SIZE,
	VS_EVT_CLONING,
	VS_EVT_COMPLETE_CLONE,
	VS_EVT_ABORT_CLONE,
	VS_EVT_STOP_CLONE,

	VS_EVT_TOTALS
};

#define REBUILD_TYPE_INIT  VS_EVT_INITIALIZING
#define REBUILD_TYPE_REGEN  VS_EVT_REBUILDING
#define REBUILD_TYPE_MIG  VS_EVT_MIGRATING
#define REBUILD_TYPE_CHECK  VS_EVT_COMPLETE_INIT
// IDE device event
enum
{
	DEV_EVT_ADDED,
	DEV_EVT_REMOVED,
	DEV_EVT_READ_ERROR,
	DEV_EVT_WRITE_ERROR,
	DEV_EVT_ATA_ECC_ERROR,
	DEV_EVT_ATA_CHANGE_MODE,
	DEV_EVT_TIMEOUT,
	DEV_EVT_MARK_FAILED,
	DEV_EVT_PCI_ERROR,
	DEV_EVT_SMART_FAILED,
	DEV_EVT_CREATE_PASS,
	DEV_EVT_MODIFY_PASS,
	DEV_EVT_DELETE_PASS,
	DEV_EVT_TOTALS
};

// SCSI bus event
enum
{
	SCSI_EVT_RESET,
	SCSI_EVT_PARITY,
	SCSI_BUS_MODE_CHANGE,
	SCSI_EVT_TOTALS
};

// Hw monitor event
enum
{
	HW_EVT_SDRAM_1BIT_ECC,
	HW_EVT_SDRAM_MULTI_BIT_ECC,
	HW_EVT_TEMP_CONTROLLER,
	HW_EVT_TEMP_BACKPLANE,
	HW_EVT_VOLTAGE15,
	HW_EVT_VOLTAGE3,
	HW_EVT_VOLTAGE5,
	HW_EVT_VOLTAGE12,
	HW_EVT_VOLTAGE1_3,
	HW_EVT_VOLTAGE2_5,
	HW_EVT_VOLTAGE1_25,
	HW_EVT_POWER1_FAILED,
	HW_EVT_FAN1_FAILED,
	HW_EVT_POWER2_FAILED,
	HW_EVT_FAN2_FAILED,
	HW_EVT_POWER3_FAILED,
	HW_EVT_FAN3_FAILED,
	HW_EVT_POWER4_FAILED,
	HW_EVT_FAN4_FAILED,
	HW_EVT_UPS_POWER_LOSS,
	// Add to log H/W monitor recovered
	HW_EVT_TEMP_CONTROLLER_R,
	HW_EVT_TEMP_BACKPLANE_R,
	HW_EVT_VOLTAGE15_R,
	HW_EVT_VOLTAGE3_R,
	HW_EVT_VOLTAGE5_R,
	HW_EVT_VOLTAGE12_R,
	HW_EVT_VOLTAGE1_3_R,
	HW_EVT_VOLTAGE2_5_R,
	HW_EVT_VOLTAGE1_25_R,
	HW_EVT_POWER1_RECOVERED,
	HW_EVT_FAN1_RECOVERED,
	HW_EVT_POWER2_RECOVERED,
	HW_EVT_FAN2_RECOVERED,
	HW_EVT_POWER3_RECOVERED,
	HW_EVT_FAN3_RECOVERED,
	HW_EVT_POWER4_RECOVERED,
	HW_EVT_FAN4_RECOVERED,
	HW_EVT_UPS_POWER_RECOVERED, 

	HW_EVT_SYSTEM_RESTARTED,
	HW_EVT_TEST,
	// added for battery backup
	HW_EVT_SYSTEM_RECOVERED,        // return from BBM
	HW_EVT_TOTALS
};

enum
{
	GHM_RECOVER			=	0,        
    GHM_OVER_TEMP		=	1,      
    GHM_FAILED			=	2,         
    GHM_UNDER_VOLTAGE	=	3,
    GHM_OVER_VOLTAGE	=	4,
    GHM_DISCOVERED		=	5,
    GHM_LOST_MIG		=	6,
    GHM_OFFLINED		=	7,
    GHM_HTTP_LOGIN		=	8,
    GHM_TELNET_LOGIN	=	9,
    GHM_VT100_LOGIN		=	10,	
    GHM_GUI_LOGIN		=	11,
    GHM_FC_LINK_UP		=	12,
    GHM_FC_LINK_DOWN	=	13,
    GHM_16BYTES_CDB		=	14,
    GHM_INIT_LBA		=	15,
    GHM_REBUILD_LBA		=	16,
    GHM_MIGRATE_LBA		=	17,
	GHM_ONE_BIT_ECC		=	18,
    GHM_MULTI_BIT_ECC	=	19,
    GHM_OFF				=	20,
    GHM_UNKNOWN			=	21,
    GHM_ADDED			=	22,
    GHM_REMOVED			=	23,
	GHM_MISMATCH		=	24,
    GHM_READY			=	25,
    GHM_RESTART			=	26,
    GHM_HALTED			=	27,
	GHM_CLONE_LBA		=	28,
	GHM_RENAMED			=	29,
	GHM_HOT_SPARE_CREATED	=	30,
	GHM_HOT_SPARE_DELETED	=	31,
	GHM_HOT_SPARE_USED		=	32,
	GHM_TOTALS	
};

#if defined(__BIG_ENDIAN__)
typedef union sSYS_TIME
{
	struct
	{
		DWORD tmFlag:1;
		DWORD tmMonth:4;
		DWORD tmYear:5;
		DWORD tmDate:5;
		DWORD tmHour:5;
		DWORD tmMinute:6;
		DWORD tmSecond:6;
	} u;
	struct
	{
		int32_t tmFlag:1;
		int32_t tmTick:31;		
	} x;
} sSYS_TIME, *pSYS_TIME;	

#else
typedef union sSYS_TIME
{
	struct
	{
		DWORD tmSecond:6;
		DWORD tmMinute:6;
		DWORD tmHour:5;
		DWORD tmDate:5;
		DWORD tmYear:5;
		DWORD tmMonth:4;
		DWORD tmFlag:1;
	} u;
	struct
	{
		int32_t tmTick:31;
		int32_t tmFlag:1;
	} x;
} sSYS_TIME, *pSYS_TIME;
// If (tmFlag == 1) ==> use YY/MM/DD/HH/MM/SS else use time tick
#endif

/* event object data structure */
typedef struct sEVET_DATA
{
	BYTE evtCategory;			// Event category
	BYTE evtType;				// Event type
	BYTE evtChannel;			// Event channel
	BYTE evtFlag;				// Additional data flag
	int32_t evtTime;			// Occurred time
	DWORD evtData1;				// Additional data1
	DWORD evtData2;				// Additional data2
	BYTE evtStr[16];			// string for event
} sEVENT_DATA, *pEVENT_DATA;	// total 32 bytes


/* strings for event category */
const static char *eventCat[] =
{
	"NO_EVENT",
	"RAIDSET",
	"VOLUMESET",
	"DEVICE",
	"HOST",
	"H/W MONITOR"
};
 
/* strings for device event */
const static char *htmDevEvent[] =
{
	"Device Inserted",
	"Device Removed",
	"Reading Error",
	"Writing Error",
	"ATA Ecc Error",
	"Change ATA Mode",
	"Time Out Error",
	"Device Failed",
	"PCI Parity Error",
	"Dev Fail (SMART)",
	"PassThrough Disk Created",
	"PassThrough Disk Modified",
	"PassThrough Disk Deleted"
};

/* strings for volumeset event */
const static char *htmVolEvent[] =
{
	"Start Initialize",
	"Start Rebuilding",
	"Start Migrating",
	"Start Checking",
	"Complete Init",
	"Complete Rebuild",
	"Complete Migrate",
	"Complete Check",
	"Create Volume",
	"Delete Volume",
	"Modify Volume",
	"Volume Degraded",
	"Volume Failed",
	"Failed Volume Revived",
	"Abort Initialization",
	"Abort Rebuilding",
	"Abort Migration",
	"Abort Checking",
	"Stop Initialization",
	"Stop Rebuilding",
	"Stop Migration",
	"Stop Checking",
	"Unsupported Stripe Size",
	"Start Cloning",
	"Complete Clone",
	"Abort Cloning",
	"Stop Cloning"
};

/* strings for raidSet event */
const static char *htmRaidEvent[] =
{
	"Create RaidSet",
	"Delete RaidSet",
	"Expand RaidSet",
	"Rebuild RaidSet",
	"RaidSet Degraded"  
};

/* strings for scsi host event */
const static char *htmScsiHostEvent[] =
{
	"SCSI Bus Reset",
	"SCSI Bus Parity",
	"SCSI Bus SE<>LVD"
};

/* strings for ide host event */
const static char *htmIdeHostEvent[] =
{
	"IDE Bus Reset",
	"IDE Bus UDMA CRC",
	"SCSI Bus SE<>LVD"
};

/* strings for hardware monitor event */
const static char *htmHwMonEvent[] =
{
	"DRAM 1-Bit ECC",					
	"DRAM Fatal Error",		

	"Controller Over Temperature",		
	"Backplane Over Temperature",		
	"1.5V Abnormal",
	"3.3V Abnormal",
	"5V Abnormal",
	"12V Abnormal",
	"1.3V Abnormal",
	"2.5V Abnormal",
	"1.2V or 1.8V or 1.25V Abnormal",
	"Power#1 Failed  ",
	"Fan#1 Failed    ",
	"Power#2 Failed  ",
	"Fan#2 Failed    ",
	"Power#3 Failed  ",
	"Fan#3 Failed    ",
	"Power#4 Failed  ",
	"Fan#4 Failed    ",
	"UPS ACPower Loss",  

	"Controller Over Temperature Recovered",
	"Backplane Over Temperature Recovered",
	"1.5V Abnormal Recovered",
	"3.3V Abnormal Recovered",
	"5V Abnormal Recovered",
	"12V Abnormal Recovered",
	"1.3V Abnormal Recovered",
	"2.5V Abnormal Recovered",
	"1.2V or 1.8V or 1.25V Abnormal Recovered",
	"Power#1 Failed Recovered",
	"Fan#1 Failed Recovered",
	"Power#2 Failed Recovered",
	"Fan#2 Failed Recovered",
	"Power#3 Failed Recovered",
	"Fan#3 Failed Recovered",
	"Power#4 Failed Recovered",
	"Fan#4 Failed Recovered",
	"UPS ACPower Loss Recovered",

	"Raid Powered On",
	"Test Event",
	"Power On With Battery Backup"
};

/* strings for new events */
const static char *htmNewEventStr[] =
{
	"Recovered",
	"Over Temp.",	
	"Failed",
	"Under Voltage",
	"Over Voltage",
	"Discovered",
	"Lost Rebuilding/Migration LBA",
	"Offlined",
	"HTTP Log In",
	"Telnet Log In",
	"VT100 Log In",
	"API Log In",
	"FC Link Up",
	"FC Link Down",
    "In U160 Mode",
    "Restart Init LBA Point",
	"Restart Rebuild LBA Point",
	"Restart Migration LBA Point",	

	"One BIT ECC Error",	
	"Multi BIT ECC Error",	
	"Off",
	"Unknown Error",	
	"Added",
	"Removed",

	// for dual controller
	"Mismatch",
	"Ready",
	"Restart",
	"Halted",
	"Restart Cloning LBA Point",
	"Renamed",
	"HotSpare Created",
	"HotSpare Deleted",
	"HotSpare Used"
};




/* event notification level*/
enum _EVT_LEVEL
{
	EVT_DISABLE = 0,
	EVT_SERIOUS,		// 1 serious event
	EVT_ERROR,			// 2 error
	EVT_WARNING,		// 3 warning
	EVT_INFO			// 4 information
};

/* event notification level for raidset */
static BYTE raidEventLevel[] =
{
	EVT_WARNING,			//RS_EVT_CREATE,
	EVT_WARNING,			//RS_EVT_DELETE,
	EVT_WARNING,			//RS_EVT_EXPAND,
	EVT_WARNING,			//RS_EVT_REBUILD,
	EVT_SERIOUS				//RS_EVT_DEGRADED,
};

/* event notification level for volumeset */
static BYTE volEventLevel[] =
{
	EVT_WARNING,			//VS_EVT_INITIALIZING,
	EVT_WARNING,			//VS_EVT_REBUILDING,
	EVT_WARNING,			//VS_EVT_MIGRATING,
	EVT_WARNING,			//VS_EVT_CHECKING,
	EVT_WARNING,			//VS_EVT_COMPLETE_INIT,
	EVT_WARNING,			//VS_EVT_COMPLETE_REBUILD,
	EVT_WARNING,			//VS_EVT_COMPLETE_MIGRATING,
	EVT_WARNING,			//VS_EVT_COMPLETE_CHECKING,
	EVT_WARNING,			//VS_EVT_CREATE,
	EVT_WARNING,			//VS_EVT_DELETE,
	EVT_WARNING,			//VS_EVT_MODIFY,
	EVT_SERIOUS,			//VS_EVT_DEGRADED,
	EVT_SERIOUS,			//VS_EVT_FAILED,
	EVT_SERIOUS,			//VS_EVT_REVIVED,

	// init/rebuild/migration ABORT & STOP event
	EVT_WARNING,			//VS_EVT_ABORT_INIT,
	EVT_WARNING,			//VS_EVT_ABORT_REBUILD,
	EVT_WARNING,			//VS_EVT_ABORT_MIGRATING,
	EVT_WARNING,			//VS_EVT_ABORT_CHECKING,
	EVT_WARNING,			//VS_EVT_STOP_INIT,
	EVT_WARNING,			//VS_EVT_STOP_REBUILD,
	EVT_WARNING,			//VS_EVT_STOP_MIGRATING,
	EVT_WARNING			//VS_EVT_STOP_CHECKING,
};

/* event notification level for disk */
static BYTE devEventLevel[] =
{
	EVT_WARNING, //DEV_EVT_ADDED,
	EVT_WARNING, //DEV_EVT_REMOVED,
	EVT_WARNING, //DEV_EVT_READ_ERROR,
	EVT_WARNING, //DEV_EVT_WRITE_ERROR,
	EVT_WARNING, //DEV_EVT_ATA_ECC_ERROR,
	EVT_WARNING, //DEV_EVT_ATA_CHANGE_MODE,
	EVT_WARNING, //DEV_EVT_TIMEOUT,
	EVT_SERIOUS, //DEV_EVT_MARK_FAILED,
	EVT_ERROR,	//DEV_EVT_PCI_ERROR,
	EVT_SERIOUS, //DEV_EVT_SMART_FAILED,
	EVT_INFO,  //DEV_EVT_CREATE_PASS,
	EVT_INFO,  //DEV_EVT_MODIFY_PASS,
	EVT_INFO,  //DEV_EVT_DELETE_PASS,
};

/* event notification level for host */
static BYTE hostEventLevel[] =
{
	EVT_INFO, //SCSI_EVT_RESET,
	EVT_INFO, //SCSI_EVT_PARITY,
	EVT_INFO //SCSI_BUS_MODE_CHANGE,
};

/* event notification level for hardware monitor */
static BYTE hwEventLevel[] =
{
	EVT_SERIOUS, //HW_EVT_SDRAM_1BIT_ECC,
	EVT_SERIOUS, //HW_EVT_SDRAM_MULTI_BIT_ECC,
	EVT_SERIOUS, //HW_EVT_TEMP_CONTROLLER,
	EVT_SERIOUS, //HW_EVT_TEMP_BACKPLANE,
	EVT_SERIOUS, //HW_EVT_VOLTAGE15,
	EVT_SERIOUS, //HW_EVT_VOLTAGE3,
	EVT_SERIOUS, //HW_EVT_VOLTAGE5,
	EVT_SERIOUS, //HW_EVT_VOLTAGE12,
	EVT_SERIOUS, //HW_EVT_VOLTAGE1_3,
	EVT_SERIOUS, //HW_EVT_VOLTAGE2_5,
	EVT_SERIOUS, //HW_EVT_VOLTAGE1_25,
	EVT_SERIOUS, //HW_EVT_POWER1_FAILED,
	EVT_SERIOUS, //HW_EVT_FAN1_FAILED,
	EVT_SERIOUS, //HW_EVT_POWER2_FAILED,
	EVT_SERIOUS, //HW_EVT_FAN2_FAILED,
	EVT_SERIOUS, //HW_EVT_POWER3_FAILED,
	EVT_SERIOUS, //HW_EVT_FAN3_FAILED,
	EVT_SERIOUS, //HW_EVT_POWER4_FAILED,
	EVT_SERIOUS, //HW_EVT_FAN4_FAILED,
	EVT_SERIOUS, //HW_EVT_UPS_POWER_LOSS,
	// Add to log H/W monitor recovered
	EVT_ERROR,	//HW_EVT_TEMP_CONTROLLER_R,
	EVT_ERROR,	//HW_EVT_TEMP_BACKPLANE_R,
	EVT_ERROR,	//HW_EVT_VOLTAGE15_R,
	EVT_ERROR,	//HW_EVT_VOLTAGE3_R,
	EVT_ERROR,	//HW_EVT_VOLTAGE5_R,
	EVT_ERROR,	//HW_EVT_VOLTAGE12_R,
	EVT_ERROR,	//HW_EVT_VOLTAGE1_3_R,
	EVT_ERROR,	//HW_EVT_VOLTAGE2_5_R,
	EVT_ERROR,	//HW_EVT_VOLTAGE1_25_R,
	EVT_ERROR,	//HW_EVT_POWER1_RECOVERED,
	EVT_ERROR,	//HW_EVT_FAN1_RECOVERED,
	EVT_ERROR,	//HW_EVT_POWER2_RECOVERED,
	EVT_ERROR,	//HW_EVT_FAN2_RECOVERED,
	EVT_ERROR,	//HW_EVT_POWER3_RECOVERED,
	EVT_ERROR,	//HW_EVT_FAN3_RECOVERED,
	EVT_ERROR,	//HW_EVT_POWER4_RECOVERED,
	EVT_ERROR,	//HW_EVT_FAN4_RECOVERED,
	EVT_ERROR,	//HW_EVT_UPS_POWER_RECOVERED,

	EVT_WARNING, //HW_EVT_SYSTEM_RESTARTED,
	EVT_SERIOUS, //HW_EVT_TEST_EVENT,
	// added for battery backup
	EVT_ERROR	 //HW_EVT_SYSTEM_RECOVERED,
};


/* event notification level for new event */
static BYTE newEventLevel[] =
{
	EVT_ERROR,			// 0 GHM_RECOVER
	EVT_SERIOUS,		// 1 GHM_OVER_TEMP
	EVT_SERIOUS,		// 2 GHM_FAILED
	EVT_SERIOUS,		// 3 GHM_UNDER_VOLTAGE
	EVT_SERIOUS,		// 4 GHM_OVER_VOLTAGE
	EVT_ERROR,			// 5 GHM_DISCOVERED
	EVT_SERIOUS,		// 6 GHM_LOST_MIG
	EVT_SERIOUS,		// 7 GHM_RESTART_LBA	// not used
	EVT_WARNING,		// 8 GHM_HTTP_LOGIN
	EVT_WARNING,		// 9 GHM_TELNET_LOGIN
	EVT_WARNING,		// 10 GHM_VT100_LOGIN
	EVT_WARNING,		// 11 GHM_GUI_LOGIN
	EVT_INFO,			// 12 GHM_FC_LINK_UP
	EVT_INFO,			// 13 GHM_FC_LINK_DOWN

	EVT_ERROR,			// 14 GHM_16BYTES_CDB 
	EVT_SERIOUS,		// 15 GHM_INIT_LBA            // restart init LBA
	EVT_SERIOUS,		// 16 GHM_REBUILD_LBA         // restart rebuild LBA
	EVT_SERIOUS,		// 17 GHM_MIGRATE_LBA         // restart migration LBA

	EVT_SERIOUS,		// 18 GHM_ONE_BIT_ECC	
	EVT_SERIOUS,		// 19 GHM_ONE_BIT_ECC
	EVT_INFO,			// 20 GHM_OFF

    EVT_SERIOUS,		// 21 GHM_UNKNOWN
    EVT_WARNING,		// 22 GHM_ADDED
    EVT_WARNING,		// 23 GHM_REMOVED
	EVT_WARNING,		// 24 GHM_MISMATCH
    EVT_INFO,			// 25 GHM_READY
    EVT_INFO,           // 26 GHM_RESTART
    EVT_SERIOUS,        // 27 GHM_HALTED
	EVT_SERIOUS,        // 28 GHM_CLONE_LBA
	EVT_INFO,			// 29 GHM_RENAMED
	EVT_WARNING,        // 30 GHM_HOT_SPARE_CREATED
	EVT_WARNING,        // 31 GHM_HOT_SPARE_DELETED
	EVT_WARNING         // 32 GHM_HOT_SPARE_USED
};


/* event level string */
const static char *eventWLevel[] =
{
	"DISABLE",
	"URGENT",		// EVT_SERIOUS
	"SERIOUS",		// EVT_ERROR
	"WARNING",		// EVT_WARNING
	"INFORMATIONAL"	// EVT_INFO
};



#define UPS_AC_POWER_LOSS	0x03
#define	UPS_OK				0x07


// Battery status
// 0--100 : battery charging status (percentage)
// 255 : battery not installed
// 254 : battery failed
#define BBM_NOT_INSTALLED	255
#define BBM_FAILED			254

///////////////////////////////////////////////////////////////////////////////////
//
// HWMON_ITEMS
// Indicating the ArcGetHWMon(...) how to retrieve hardware information.
//
enum
{
	HWMON_FAN_INFO = 0x01,
	HWMON_VOL_INFO,
	HWMON_TMP_INFO,
	HWMON_PWR_INFO,
	HWMON_UPS_INFO,
	HWMON_BATTERY_INFO,	//BBM(Battery Backup Module)
	HWMON_TOTAL
};

typedef struct _HWMON_INFO
{
	WORD	HwItemCount;	// item count of hardware( including fans, voltages, temperatures, powers)
	WORD	ItemInfo[50];	// item information
}sHwMonInfo, *pHwMonInfo;
///////////////////////////////////////////////////////////////////////////////////
//
//	Hardware monitor information for raid controller excepts ARC-5010 & ARC-6010.
//  ARC-5010 & ARC-6010 should use the "sHwMon5010" data structure instead.
//	Because the number of fans, voltages, temperatures and powers are differ from 
//	models of areca except the ARC-5010 & ARC-6010, so we use the flexible data 
//	structure instead for different models of areca.
//
//  Pass the flag above & pointer of the struct _HWMON_INFO to the ArcGetHWMon(...) to 
//  retrieve the hardware information.
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//  For example: Retrieving hardware monitor of fans
//
//	CArclib al;
//  ARC_STATUS status;
//  sHwMonInfo	HwInfo;
//  
//	WinCommInterface *iface = new WinCommInterface();
//	if(!iface->init(0))	// init "COM1"
//		return;
//	status = al.ArcInitSession(iface);
//  if(status != ARC_SUCCESS)
//  {
//      .... handle error .....
//  }
//  status = al.ArcGetHWMon(HWMON_FAN_INFO,  &HwInfo);
//  
//  if the status = ARC_SUCCESS, then the contents of HwInfo are:
//  HwInfo.HwItemCount ==> # of fans
//  HwInfo.ItemInfo[0] ==> RPM of 1st fan
//	HwInfo.ItemInfo[1] ==> RPM of 2nd fan
//   ...
//  HwInfo.ItemInfo[HwInfo.HwItemCount - 1] ==> RPM of the last fan
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//	For example: Retrieving hardware monitor of voltage	
//	status = al.ArcGetHWMon(HWMON_VOL_INFO,  &HwInfo);
//	if the status = ARC_SUCCESS, then the contents of HwInfo are:
//	HwInfo.HwItemCount ==> # of voltage items
//	HwInfo.ItemInfo[0] ==> base value of 1st voltage
//	HwInfo.ItemInfo[1] ==> current value of 1st voltage
//	HwInfo.ItemInfo[2] ==> base value of 2nd voltage
//	HwInfo.ItemInfo[3] ==> current value of 2nd voltage
//	...
//	HwInfo.ItemInfo[HwInfo.HwItemCount * 2 - 2] ==> base value of last voltage
//	HwInfo.ItemInfo[HwInfo.HwItemCount * 2 - 1] ==> current value of last voltage
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//	For example: Retrieving hardware monitor of temperature	
//	status = al.ArcGetHWMon(HWMON_TMP_INFO,  &HwInfo);
//	if the status = ARC_SUCCESS, then the contents of HwInfo are:
//	HwInfo.HwItemCount ==> # of temperature items
//	HwInfo.ItemInfo[0] ==> controller temperature
//	HwInfo.ItemInfo[1] ==> 1st HDD temperature
//	HwInfo.ItemInfo[2] ==> 2nd HDD temperature
//	...
//	HwInfo.ItemInfo[HwInfo.HwItemCount - 1] ==> last temperature of HDD
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//	For example: Retrieving hardware monitor of power	
//	status = al.ArcGetHWMon(HWMON_PWR_INFO,  &HwInfo);
//	if the status = ARC_SUCCESS, then the contents of HwInfo are:
//	HwInfo.HwItemCount ==> # of power items
//	HwInfo.ItemInfo[0] ==> 1st power state
//	HwInfo.ItemInfo[1] ==> 2nd power state
//	...
//	HwInfo.ItemInfo[HwInfo.HwItemCount - 1] ==> last power state
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//  Note: Areca specifications
//	For fans:  the fan speed != 0 means fan ok else failed, if the speed == 0xffff then there is 
//             no fan connected.
//	For voltages: one voltage has pair values, one for base value(fixed), the other for current 
//				  voltage value		   
//				  if ( current value > base value * 1.1) or (current value < base value * 0.9)
//				  then the voltage is abnormal.
//  For temperature: Including Controller & HDD temperature, the 1st value is controller temperature 
//					 and the others are HDD temperatures.
//					 If the value over 60 degrees of centigrade, then the temperature is abnormal.
//  For power:	The value is "1" means the power Ok , "0" means failed.
//


///////////////////////////////////////////////////////////////////////////////////
//
//	Hardware monitor data structure for 5 bays machines only (ARC-5010 & ARC-6010).
//	Since the number of fans, voltages, temperatures and powers of the ARC-5010 & ARC-6010 
//  are fixed, so we can use the "fixed" data structure to retrieve hardware information.
//
typedef struct _HWMON_5010
{	
	BYTE	fans;			// # of fans
	BYTE	voltages;		// # of voltages
	BYTE	temps;			// # of temperatures ( only Controller & BackPlane )
	BYTE	powers;			// # of powers
	WORD	monFan;			// fan speed(in RPM)
	WORD	fixed12V;		// always 12000mV(12V)
	WORD	mon12V;			// current voltage of 12000mV
	WORD	fixed5V;		// always 5000mV(5V)
	WORD	mon5V;			// current voltage of 5000mV
	WORD	fixed3_3V;		// always 3300mV(3.3V)
	WORD	mon3_3V;		// current voltage of 3300mV
	WORD	fixed1_5V;		// always 1500mV(1.5V)
	WORD	mon1_5V;		// current voltage of 1500mV
	BYTE	monCtrlTemp;	// current temperature of contoller(in degrees centigrade)
	BYTE	monBkPlaneTemp;	// current temperature of backplane(in degrees centigrade)
	BYTE	reserved[2];	// not use
} sHwMon5010, *pHwMon5010;



///////////////////////////////////////////////////////////////////////////////////
//
//	Volumeset capabilities pass into ArcCreateVolSet(...)
//
typedef struct _VOLUMESET_CAP
{
	DWORD	structs_size;	// size of _VOLUMESET_CAP structure
	BYTE	raid;			// RAIDSet Number that volumeset belongs to.
	char	volname[16];	// volume Set Name
	double	capacity;		// volume capacity size ( in GBs )	
	BYTE	raidlevel;		// raid level
	BYTE	stripsize;		// stripe size (0/1/2/3/4/5->4/8/16/32/64/128K)
	BYTE	channel;		// channel#
	BYTE	id;				// ID#
	BYTE	lun;			// LUN#
	BOOL	tag;			// Enable tag queue(1 : enabled)
	BOOL	cache;			// Enable cache(1 : enabled)
	BYTE	speed;			// speed (0/1/2/3/4->async/20/40/80/160 for scsi)
							// (0/1/2/3/4->33/66/100/133/150 for IDE)
	BYTE	quickinit;		// 0:background init, 1:foreground init
	BYTE    encypttype;		// 0:no encryption
                            // 1:256 bit/password
                            // 2:256 bit/AES
                            // 3:128 bit/password
                            // 4:128 bit/AES
}sVolumeSetCap, *pVolumeSetCap;		

///////////////////////////////////////////////////////////////////////////////////
//
//	New volumeset capabilities pass into ArcModifyVolSet(...)
//
typedef struct _NEW_VOLUMESET_CAP
{
	DWORD	structs_size;		// size of _NEW_VOLUMESET_CAP structure
	BYTE	vol;				// VolumeSet Number
	char	newVolname[16];		// volumeSet Name
	double	newCapacity;		// volume capacity size in GBs (reserved)
	BYTE	newRaidlevel;		// raid level
	BYTE	newStripsize;		// stripe size (0/1/2/3/4/5->4/8/16/32/64/128K)
	BYTE	newChannel;			// channel#
	BYTE	newId;				// ID#
	BYTE	newLun;				// LUN#
	BYTE	newTag;				// Enable tag queue(1 : enabled)
	BYTE	newCache;			// Enable cache(1 : enabled)
	BYTE	newSpeed;			// speed (0/1/2/3/4->async/20/40/80/160 for scsi)
								// (0/1/2/3/4->33/66/100/133/150 for IDE)
}sNewVolumeSetCap, *pNewVolumeSetCap;		

///////////////////////////////////////////////////////////////////////////////////
//
//	Pass-through disk capabilities pass into ArcCreatePassthroughDisk(...)
//

#define ATA33			0
#define ATA66			1
#define ATA100			2
#define ATA133			3
#define ATA150			4

typedef struct _PASS_DISK_CAP
{
	DWORD	structs_size;	// size of _PASS_DISK_CAP structure
	BYTE	nDevNum;		// device #
	BYTE	nScsiCh;		// scsi channel (0/1)
	BYTE	nScsiId;		// scsi id (0-->15)
	BYTE	nScsiLun;		// scsi lun (0-->7)
	BYTE	nTaggedQue;		// tagged queue (1 : enabled)
	BYTE	nCache;			// cache mode (1 : enabled)
	BYTE	nMaxSpeed;		// (0/1/2/3/4, async/20/40/80/160 for scsi)
							// (0/1/2/3, 33/66/100/133 for ide)
	                        
}sPassDiskCap, *pPassDiskCap;


typedef struct _VOL_MOD
{
	BYTE	num_vol;		// volumeset# to modify 
	BYTE	newlevel;		// new raid level of volumeset
	BYTE	newstripesize;  // new stripe size of volumeset
	BYTE	reserved;		// reserved
}sVolMod, *pVolMod;

typedef struct _VOL_MOD_LIST
{
	BYTE	total_vol;		// specifies how many volumesets to modify
	sVolMod	vol_list[128];	// 
}sVolModList, *pVolModList;


#ifndef		OID
#define		OID		long
#endif


typedef struct _NET_INFO
{
	BYTE	dhcp;					// 1 for enabled, 0 for disabled
	BYTE	local_ip[4];			// for static IP
	BYTE	gateway_ip[4];			// for static gateway
	BYTE	subnet_mask[4];			// for static subnet mask
	WORD	http_port;				// current http port#
	WORD	telnet_port;			// current telnet port#
	WORD	smtp_port;				// current smtp port#	
	BYTE    current_ip[4];			// current IP address
	BYTE	mac_addr[6];			// MAC address
	BYTE	reserved[3];
}sNET_INFO, *pNET_INFO;

typedef struct _SYS_CONFIG_INFO
{
	BYTE	beeper;							// beeper setting
	BYTE	bgtask_priority;				// background task priority setting
	BYTE	jbod_raid;						// JBOD/RAID setting
	BYTE	hdd_read_ahead;					// HDD read ahead cache setting
	BYTE	stagger_power_on;				// stagger power on setting
	BYTE	auto_spin_down;					// hdd spin down mode
	BYTE    hdd_low_power_idle_mode;        // hdd low power idle mode
	BYTE    hdd_low_rpm_mode;				// hdd low RPM mode
	BYTE	empty_hdd_slot_led;				// empty hdd slot led setting
	BYTE	cpu_fan_detect;					// cpu fan detect
	BYTE	hdd_smart_status_polling;		// hdd smart status polling
	BYTE    sas_mux;                        // SAS Mux
	BYTE    ses2_support;                   // SES2 Support
	BYTE	auto_activate_incomplete_raid;	// auto activate incomplete raid
	BYTE	disk_write_cache;				// disk write cache
	BYTE	disk_capacity_truncation;		// disk capacity truncation
	BYTE	sata_ncq_support;				// SATA NCQ support
	BYTE	vol_readahead;					// volume data read ahead
	BYTE	hdd_queue_depth;				// HDD queue depth
	BYTE    max_cmd_len;					// Max Command Length
	BYTE    write_same_for_init;            // Write same for initialization
	BYTE    hotplugged_for_rebuilding;      // Hot Plugged disk for rebuilding
	BYTE    smart_option_for_hdd;           // Smart option for HDD
	BYTE    Smart_polling_interval;         // Smart polling interval
	BYTE    reserved[8];
}sSYS_CONFIG_INFO, *pSYS_CONFIG_INFO;


//	SMTP SUPPORTED
typedef struct _MAIL_ADDR
{
	BYTE    mailName[20];		    // name
	BYTE    mailAddress[50];	    // mail address, format: abc@company.com
} sMAIL_ADDR, *pMAIL_ADDR;

typedef struct _MAIL_CONFIG
{
	BYTE				ip[4];			// smtp server ip
	BYTE				account[50];	// account
	BYTE				password[20];	// password	
	sMAIL_ADDR          sender;			// 1 sender address
	sMAIL_ADDR          mailto[4];		// 4 mailto address
	DWORD				eventlevel[4];		// event notification level(MAIL)
	DWORD				notificationfornoevent[4];	// notify user if no event occurs within 24 hours
}sMAIL_INFO, *pMAIL_INFO;



// SNMP SUPPORTED
typedef struct _SNMP_CONFIG
{
	BYTE    ip[3][4];		// snmp trap ip 1, 2, 3
	WORD	port[3];		// snmp trap port 1, 2, 3
	BYTE	community[16];  // snmp community
	BYTE    contact[32];	// snmp sys contact
	BYTE    location[32];	// snmp sys location
	BYTE    name[32];		// snmp sys name
	DWORD	eventlevel;		// event notification level(SNMP)
	BYTE    path;
	BYTE	reserved[1];	// reserved
}sSNMP_CONFIG, *pSNMP_CONFIG;


/*  TIME ZONE DEFINITIONS
0 = (GMT-12:00)International Date Line West
1 = (GMT-11:00)Midway Island, Samoa
2 = (GMT-10:00)Hawaii 
3 = (GMT-09:00)Alaska 
4 = (GMT-08:00)Pacific Time(US&Canada), Tijuana 
5 = (GMT-07:00)Arizona 
6 = (GMT-07:00)Chihuahua, La Paz, Mazatlan 
7 = (GMT-07:00)Mountain Time(US&Canada) 
8 = (GMT-06:00)Central America 
9 = (GMT-06:00)Central Time(US&Canada) 
10 = (GMT-06:00)Guadalajara, Mexico City, Monterrey 
11 = (GMT-06:00)Saskatchewan 
12 = (GMT-05:00)Bogota, Lima, Quito 
13 = (GMT-05:00)Eastern Time(US&Canada) 
14 = (GMT-05:00)Indiana(East) 
15 = (GMT-04:00)Atlantic Time(Canada) 
16 = (GMT-04:00)Caracas, La Paz 
17 = (GMT-04:00)Santiago 
18 = (GMT-03:30)Newfoundland 
19 = (GMT-03:00)Brasilia 
20 = (GMT-03:00)Buenos Aires, Georgetown 
21 = (GMT-03:00)Greenland 
22 = (GMT-02:00)Mid-Atlantic 
23 = (GMT-01:00)Azores 
24 = (GMT-01:00)Cape Verde Is. 
25 = (GMT)Casablanca, Monrovia 
26 = (GMT)Greenwich Mean Time, Dublin, Edinburgh, Lisbon, London 
27 = (GMT+01:00)Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna 
28 = (GMT+01:00)Belgrade, Bratislava, Budapest, Ljubljana, Prague 
29 = (GMT+01:00)Brussels, Copenhagen, Madrid, Paris 
30 = (GMT+01:00)Sarajevo, Skopje, Warsaw, Zagreb 
31 = (GMT+01:00)West Central Africa 
32 = (GMT+02:00)Athens, Beirut, Istanbul, Minsk 
33 = (GMT+02:00)Bucharest 
34 = (GMT+02:00)Cairo 
35 = (GMT+02:00)Harare, Pretoria 
36 = (GMT+02:00)Helsinki, Kyiv, Riga, Sofia, Tallinn, Vilnius 
37 = (GMT+02:00)Jerusalem 
38 = (GMT+03:00)Baghdad 
39 = (GMT+03:00)Kuwait, Riyadh 
40 = (GMT+03:00)Moscow, St. Petersburg, Volgograd 
41 = (GMT+03:00)Nairobi 
42 = (GMT+03:30)Tehran 
43 = (GMT+04:00)Abu Dhabi, Muscat 
44 = (GMT+04:00)Baku, Tbilisi, Yerevan 
45 = (GMT+04:30)Kabul 
46 = (GMT+05:00)Ekaterinburg 
47 = (GMT+05:00)Islamabad, Karachi, Tashkent 
48 = (GMT+05:30)Chennai, Kolkata, Mumbai, New Delhi 
49 = (GMT+05:45)Kathmandu 
50 = (GMT+06:00)Almaty, Novosibirsk 
51 = (GMT+06:00)Astana, Dhaka 
52 = (GMT+06:00)Sri Jayawardenepura 
53 = (GMT+06:30)Rangoon 
54 = (GMT+07:00)Bangkok, Hanoi, Jakarta 
55 = (GMT+07:00)Krasnoyarsk 
56 = (GMT+08:00)Beijing, Chongging, Hong Kong, Urumqi 
57 = (GMT+08:00)Irkutsk, Ulaan Bataar 
58 = (GMT+08:00)Kuala Lumpur, Singapore 
59 = (GMT+08:00)Perth 
60 = (GMT+08:00)Taipei 
61 = (GMT+09:00)Osaka, Sapporo, Tokyo 
62 = (GMT+09:00)Seoul 
63 = (GMT+09:00)Yakutsk 
64 = (GMT+09:30)Adelaide 
65 = (GMT+09:30)Darwin 
66 = (GMT+10:00)Brisbane 
67 = (GMT+10:00)Canberra, Melbourne, Sydney 
68 = (GMT+10:00)Guam, Port Moresby 
69 = (GMT+10:00)Hobart 
70 = (GMT+10:00)Vladivostok 
71 = (GMT+11:00)Magadan, Solomon Is., New Caledonia 
72 = (GMT+12:00)Auckland, Wellington 
73 = (GMT+12:00)Fiji, Kamchatka, Marshall Is. 
74 = (GMT+13:00)Nuku'alofa 

*/

typedef struct _NTP_CONFIG
{
	BYTE ip[2][4];			// ntp server ip 1,2
	BYTE timezone;			// timezone, see above
	BYTE autodls;			// automatic daylight saving
	BYTE reserved[2];		// reserved
}sNTP_CONFIG, *pNTP_CONFIG;


typedef struct _DEVICE_LIST
{
	int	total;
	struct
	{
		BYTE nEncIndex;	// the Enclosure index, for non-SAS controller this field is always set to '0'
		BYTE nDrvIndex;	// the Drive index
	}device_pair[320];
}sDEVICE_LIST, *pDEVICE_LIST;


typedef struct _DRV_MAP
{
	int total;
	struct
	{
		BYTE encindex;
		BYTE drvindex;	
		sGUI_PHY_DRV drvInfo;
	}drv[320];
}sDRV_MAP, *pDRV_MAP;
	
typedef struct _VOL_MAP
{
	int total;
	struct
	{
		DWORD volindex;
		sGUI_VOLUMESET volInfo;
	}vol[128];
}sVOL_MAP, *pVOL_MAP;

typedef struct _RAID_MAP
{
	int total;
	struct
	{
		DWORD raidindex;
		sGUI_RAIDSET raidInfo;
	}raid[128];
}sRAID_MAP, *pRAID_MAP;


//ATA passthrough (read) with data :
// a. returns 512 bytes of data:
//   1. ATA_SMART_CMD(ATA_SMART_READ_VALUES)
//   2. ATA_SMART_CMD(ATA_SMART_READ_THRESHOLDS)
//   3. ATA_SMART_CMD(ATA_SMART_READ_LOG_SECTOR)
//   4. ATA_IDENTIFY_DEVICE
//   5. ATA_IDENTIFY_PACKET_DEVICE
// b. returns 1 byte of data
//   1. ATA_CHECK_POWER_MODE,
//ATA passthrough (write) with data :
// a. writes 512 bytes of data:
//   1. ATA_SMART_CMD(ATA_SMART_WRITE_LOG_SECTOR)
//ATA passthrough (no data) without data :
//   1. ATA_SMART_CMD(ATA_SMART_ENABLE)
//   2. ATA_SMART_CMD(ATA_SMART_DISABLE)
//   3. ATA_SMART_CMD(ATA_SMART_AUTOSAVE)
//   4. ATA_SMART_CMD(ATA_SMART_IMMEDIATE_OFFLINE)
//   5. ATA_SMART_CMD(ATA_SMART_AUTO_OFFLINE)
//   6. ATA_SMART_CMD(ATA_SMART_STATUS)
//      6.1 STATUS,       // SMART is working or not
//      6.2 STATUS_CHECK, // SMART status is healthy(0) or failing(1)

// ATA commands
#define ATA_IDENTIFY_DEVICE             0xec                                              
#define ATA_IDENTIFY_PACKET_DEVICE      0xa1
#define ATA_SMART_CMD                   0xb0
#define ATA_CHECK_POWER_MODE            0xe5
#define ATA_DOWNLOAD_MICROCODE          0x92

// ATA feature register values (ATA_SMART_CMD subcommands).
#define ATA_SMART_READ_VALUES           0xd0
#define ATA_SMART_READ_THRESHOLDS       0xd1
#define ATA_SMART_AUTOSAVE              0xd2
#define ATA_SMART_SAVE                  0xd3
#define ATA_SMART_IMMEDIATE_OFFLINE     0xd4
#define ATA_SMART_READ_LOG_SECTOR       0xd5
#define ATA_SMART_WRITE_LOG_SECTOR      0xd6
//#define ATA_SMART_WRITE_THRESHOLDS      0xd7
#define ATA_SMART_ENABLE                0xd8
#define ATA_SMART_DISABLE               0xd9
#define ATA_SMART_STATUS                0xda
#define ATA_SMART_AUTO_OFFLINE          0xdb

// Sector number values for ATA_SMART_IMMEDIATE_OFFLINE subcommand
#define OFFLINE_FULL_SCAN               0
#define SHORT_SELF_TEST                 1
#define EXTEND_SELF_TEST                2
#define CONVEYANCE_SELF_TEST            3
#define SELECTIVE_SELF_TEST             4
#define ABORT_SELF_TEST                 127
#define SHORT_CAPTIVE_SELF_TEST         129
#define EXTEND_CAPTIVE_SELF_TEST        130
#define CONVEYANCE_CAPTIVE_SELF_TEST    131
#define SELECTIVE_CAPTIVE_SELF_TEST     132
#define CAPTIVE_MASK                    (0x01<<7)

#define FI_NONE				0x00
#define FI_OFFSET_NEW		0x01
#define FI_OFFSET_ADD		0x02
#define FI_OFFSET_SUB		0x04
#define FI_SIZE_NEW			0x08
#define FI_SIZE_ADD			0x10
#define FI_SIZE_SUB			0x20
	
class CFieldInfo
{
public:
	CFieldInfo()
	{
		offset = 0;
		datalen = 0;
		flag = FI_NONE;
	};
	long offset;	// offset of field
	long datalen;	// data length of field
	DWORD flag;		// flag
};

///////////////////////////////////////////////////////////////////////////////////
//
//	CArclib definitions
//	This class is exported from the Areca GUI library
// 
#ifdef _WIN32
class ARCLIB_API CArclib : public CArclibBase
{
#else
class CArclib : public CArclibBase
{		
#endif
// public constructors & destructors
public:
	CArclib();
	~CArclib();

// public member functions for raidsubsystem
public:
	/************************************************************************************
	Function Name:	ArcGetLibVersion
	Function:		Get the areca library version
	Input:			None.
	Output:			None.
	Return Values:	1. Library version
	Note:			None.
	************************************************************************************/
	static int ArcGetLibVersion
	(
		void
	);

	virtual ArcInterface *ArcGetInterface();
	/************************************************************************************
	Function Name:	ArcInitSession
	Function:		Startup areca library by assigning an interface to it
	Input:			1. ArcInterface
	Output:			None.
	Return Values:	1. ARC_STATUS					
	Note:			This function should be called before any member functions be called
	************************************************************************************/	
	virtual ARC_STATUS ArcInitSession
	(
		IN ArcInterface *Interface
	);

	/************************************************************************************
	Function Name:	ArcGetSysInfo
	Function:		Get the RAID system information
	Input:			None.
	Output:			1. Pointer to a sSYSTEM_INFO structure that receives the information.					   
	Return Values:	1. ARC_STATUS
	Note:			1. The memory allocation of the sSYSTEM_INFO structure is the 
	                   responsibility of the calling function.		
	************************************************************************************/
	virtual ARC_STATUS ArcGetSysInfo
	(
		OUT pSYSTEM_INFO  pSysInfo
	);

	/************************************************************************************
	Function Name:	ArcGetRaidSetInfo
	Function:		Get the RaidSet information
	Input:			1. RaidSet number 					
	Output:			1. Pointer to a sGUI_RAIDSET structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. RaidSet number is started from 0 to sSYSTEM_INFO.gsiMaxRaidSet
	                2. The memory allocation of the sGUI_RAIDSET structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetRaidSetInfo
	(
		IN	BYTE iRaidNumber, 
		OUT pGUI_RAIDSET pRaidInfo		
	);

	/************************************************************************************
	Function Name:	ArcGetVolSetInfo
	Function:		Get the VolumeSet information
	Input:			1. VolumeSet number					
	Output:			1. Pointer to a sGUI_VOLUMESET structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. VolumeSet number is started from 0 to sSYSTEM_INFO.gsiMaxVolumeSet
	                2. The memory allocation of the sGUI_VOLUMESET structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetVolSetInfo
	(
		IN	BYTE iVolNumber, 
		OUT pGUI_VOLUMESET pVolSet
	);

	/************************************************************************************
	Function Name:	ArcGetPhysicalDriveInfo
	Function:		Get the physical drive information
	Input:			1. nDrvNum: Physical dirve number	                
	Output:			1. Pointer to a sGUI_PHY_DRV structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. Drive number is started from 0 to sSYSTEM_INFO.gsiIdeChannels
	                2. The memory allocation of the sGUI_PHY_DRV structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetPhysicalDriveInfo
	(
		IN	BYTE nDrvNum, 
		OUT	pGUI_PHY_DRV pPhyDrv
	);	

	/************************************************************************************
	Function Name:	ArcGetHWMon
	Function:		Get the H/W monitor status
	Input:			1. Hardware monitor item 
	Output:			1. Pointer to a sHwMonInfo structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. HwItem is described by HWMON_ITEMS
	                2. The memory allocation of the sHwMonInfo structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetHWMon
	( 
		IN	DWORD HwItem, 
		OUT	pHwMonInfo pHwInfo 
	);

	/************************************************************************************
	Function Name:	ArcGetHWMon5010
	Function:		Get the H/W monitor status for 5010 & 6010 controller only
	Input:			None.
	Output:			1. Pointer to a sHwMon5010 structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. The memory allocation of the sHwMon5010 structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetHWMon5010
	( 
		OUT pHwMon5010 pHwMon 
	);

	/************************************************************************************
	Function Name:	ArcSetSerial
	Function:		Set serial number of RAID subsystem
	Input:			1. Serial number(null terminate), Max 16 characters
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetSerial
	(
		IN char *szSerialNumber
	);

	/************************************************************************************
	Function Name:	ArcSetVendor
	Function:		Set vendor string of RAID subsystem
	Input:			1. Vendor string(null-terminated), Max 40 characters
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetVendor
	(
		IN char *szVendorName
	);

	/************************************************************************************
	Function Name:	ArcSetModel
	Function:		Set model string of RAID subsystem
	Input:			1. Model string(null-terminated), Max 8 characters
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetModel
	(
		IN char *szModelName
	);

	/************************************************************************************
	Function Name:	ArcGetIdentifyString
	Function:		Get controller identify string
	Input:			None.
	Output:			1. Pointer to a buffer that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. The memory allocation of the buffer is the responsibility of the 
	                   calling function and the buffer must be >= 44 bytes 
	************************************************************************************/
	virtual ARC_STATUS ArcGetIdentifyString
	(
		OUT char *strBuf
	);

	/************************************************************************************
	Function Name:	ARC_STATUS ArcIdentify(void)
	Function:		Identify controller
	Input:			None
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			1. If ARC_STATUS == ARC_SUCCESS then the controller found else
	                   no controller found.
	************************************************************************************/
	virtual ARC_STATUS ArcIdentify
	(
		void
	);

	/************************************************************************************
	Function Name:	ArcMuteBeeper
	Function:		Mute the beeper
	Input:			None
	Output:			None
	Return Values:	1. ARC_STATUS 				
	Note:			1. Password required if password enabled, use ArcChkPassword(...)
					   to validate the password.						
	************************************************************************************/
	virtual ARC_STATUS ArcMuteBeeper
	( 
		void 
	);

	/************************************************************************************
	Function Name:	ArcSetBeeper
	Function:		Set the Beeper state
	Input:			1. DISABLE or ENABLE
	Output:			None
	Return Values:	1. ARC_STATUS  				
	Note:			1. Password required if password enabled, use ArcChkPassword(...)
					   to validate the password.
	************************************************************************************/
	virtual ARC_STATUS ArcSetBeeper
	(
		IN BYTE bSetting
	);

	/************************************************************************************
	Function Name:	ArcSetMacAddr
	Function:		Set MAC Address of RAID subsystem if ethernet supported
	Input:			1. Standard MAC Address format, must be 6 byte characters.
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			1. Example of MAC Address format, 
	                   char mac[6] = { 0x00, 0x04, 0xD9, 0x60, 0x00, 0x00 };
	************************************************************************************/
	virtual ARC_STATUS ArcSetMacAddr
	(
		IN char *MacAddress
	);

	/************************************************************************************
	Function Name:	ArcSetLogo
	Function:		Set the OEM logo
	Input:			1. The buffer contains the context of the image( .jpg or .gif )
					2. The actually size of image, not the buffer size.
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			1. Image size must be <= 7998 bytes
					2. The sequences of set logo:
					   2a. Read the image file from disk into the buffer that calling
					       function allocated.
					   2b. Pass the buffer and image size as the parameter into the 
					       member function ArcSetLogo(...)
	************************************************************************************/
	virtual ARC_STATUS ArcSetLogo
	(
		IN char *imageBuf, 
		IN int imageSize
	);

	/************************************************************************************
	Function Name:	ArcSetTime
	Function:		The ArcSetTime method synchronizes the RAID subsystem data-time with 
	                the host date-time.
	Input:			1. Flag 0: Use local time to synchronize the RAID subsystem 
	                        1: Use UTC format to synchronize the RAID subsystem, 
							   also update the RAID subsystem real-time clock (RTC)
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			None
	************************************************************************************/
	virtual ARC_STATUS ArcSetTime
	(
		BYTE flag = 0 
	);

	/************************************************************************************
	Function Name:	ArcSNMP
	Function:		Query SNMP value with SNMP standard message ( BER encoding )
	Input:			1. SNMP Message ( BER encoding )
					2. Length of SNMP Message
	Output:			1. SNMP Message returned from RAID subsystem ( BER encoding )
					2. The length of returning SNMP message
	Return Values:	1. ARC_STATUS
	Note:			1. This member function is available if sSYSTEM_INFO.gsiRs232Snmp were true.
	************************************************************************************/
	virtual ARC_STATUS ArcSNMP
	(
		IN	char *SNMPMsg, 
		IN	int length, 
		OUT	char *SNMPResponse, 
		OUT	int *ResLength
	);

	/************************************************************************************
	Function Name:	ArcSNMPWithOid
	Function:		Query SNMP value with OID name
	Input:			1. The OID from the NMS Request
					2. Length of OID( both OID-IN and OID-OUT )
					3. Community string
					4. Exact ( if exact == 1 then SNMPGET else SNMPGETNEXT  )
	Output:			1. The OID returned from RAID-Subsystem 
					2. SNMP tag of the returning value
					3. The answer of this OID
					4. The length of this answer
	Return Values:	1. ARC_STATUS
	Note:			1. This member function is available if sSYSTEM_INFO.gsiRs232Snmp were true.
	                2. That length serve as a dual purpose in both IN and OUT.
					3. The calling thread has the responsibility of freeing memory of var
					   which allocates by ArcSNMPWithOid(...)
	************************************************************************************/
	virtual ARC_STATUS ArcSNMPWithOid
	(
		IN	OID	*oid, 
		IN OUT int	*length, 
		IN  unsigned char *szCommunity,
		OUT OID	*ret_oid, 
		OUT	unsigned char *tag, 
		OUT	unsigned char **var,
		OUT	int		*var_len,
		IN unsigned char exact = 0 // default SNMPGETNEXT
	);

	/************************************************************************************
	Function Name:	ArcSetPassword
	Function:		Set the password of the RAID Subsystem
	Input:			1. New password with null-terminated
	Output:			None.  
	Return Values:	1. ARC_STATUS
	Note:			1. New password string must be <= 15 characters	
	************************************************************************************/
	virtual ARC_STATUS ArcSetPassword
	(
		IN char *szNewPasswordString
	);  

	/************************************************************************************
	Function Name:	ArcSetHostMode
	Function:		Set host mode
	Input:			0->Independent(RAID), 1->cluster(JBOD)
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetHostMode
	(
		IN BYTE bHostMode
	);

	/************************************************************************************
	Function Name:	ArcSetRebuildPriority
	Function:		Rebuild priority setting
	Input:			0/1/2/3 (low->high)
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetRebuildPriority
	(
		IN BYTE bPriority
	);

	/************************************************************************************
	Function Name:	ArcSetMaxAtaMode
	Function:		Set max ATA or SATA mode
	Input:			IDE_SPEED_33 / IDE_SPEED_66 / IDE_SPEED_100 / IDE_SPEED_133 for IDE
	                SATA_SPEED_150 / SATA_SPEED_150NCQ / SATA_SPEED_300 / SATA_SPEED_300NCQ for SATA
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetMaxAtaMode
	(
		IN BYTE bMode
	);

	/************************************************************************************
	Function Name:	ArcSetDhcpIp
	Function:		Set DHCP and IP
	Input:			1. 0:dhcp disabled, 1:dhcp enabled
					2. Null-terminated character string representing a number expressed in the Internet standard "."
					   (dotted) notation, format: "xxx.xxx.xxx.xxx" ex: 192.168.001.100
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetDhcpIp
	(
		IN BYTE bDhcp, 
		IN PBYTE szIp
	);

	/************************************************************************************
	Function Name:	ArcSetEthernetMask
	Function:		Set EtherNet Mask IP
	Input:			1. Null-terminated character string representing a number expressed in the Internet standard "."
					   (dotted) notation, format: "xxx.xxx.xxx.xxx" ex: 192.168.001.100
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS	ArcSetEthernetMask
	(
		IN char *szMaskIp
	);


	/************************************************************************************
	Function Name:	ArcSetRaidTermPort
	Function:		Set Raid Terminal Port
	Input:			1. 0->COMA (term port), 1->COMB (debug port)
					2. 0/1/2/3/4/5/6/7 (1200/2400/4800/9600/19200/38400/57600/115200)
					3. stop bit (0:1, 1:2 stop bits)
	Output:			None.  
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetTermPort
	(
		IN BYTE nPort, 
		IN BYTE nBaudRate, 
		IN BYTE nStopBit
	);

	/************************************************************************************
	Function Name:	ArcChkPassword
	Function:		Pass the password that user input to RAID Subsystem
	Input:			1. Password with null-terminated
	Output:			None.	
	Return Values:	1. ARC_STATUS
	Note:			1. Password string must be <= 15 characters	
	************************************************************************************/
	virtual ARC_STATUS ArcChkPassword
	(
		IN char *szPassword
	);

	/************************************************************************************
	Function Name:	ArcLogOut
	Function:		Logout GUI (force password checking on next command)
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcLogOut
	( 
		void 
	);

	/************************************************************************************
	Function Name:	ArcHttp
	Function:		Translates the HTTP data between BROWSER and RAID Subsystem
	Input:			1. HTTP data from browser( HTML format )
					2. The length of the HTTP data from browser
	Output:			1. The HTTP data returned from RAID subsystem ( HTML format)
					2. The Length of the returning HTTP data
	Return Values:	1. ARC_STATUS
	Note:			1. Sequence of translation.
					   1a. Setup the sockets for waiting connection from remote side.
					   1b. Receive the HTTP data from remote side.
					   1c. Pass the received data and length into ArcHttp.
					   1d. Send the HTTP data returned from RAID subsystem back to remote side.
	************************************************************************************/
	virtual ARC_STATUS ArcHttp
	(
		IN	char *pClientHttpData, 
		IN	int nClientHttpDataSize, 
		OUT	char *pRaidReturnHttp, 
		OUT	int *nRaidReturnSize
	);

	/************************************************************************************
	Function Name:	ArcPollEvent
	Function:		Get the current index of event object in the RAID subsystem
	Input:			None.
	Output:			1. Pointer to an INTEGER to save the index of current event object
	Return Values:	1. ARC_STATUS
	Note:			
	************************************************************************************/
	virtual ARC_STATUS ArcPollEvent
	(
		OUT int *nCurrentEvtIndex
	);

	/************************************************************************************
	Function Name:	ArcGetReqEventPage
	Function:		Get the specific number of event pages from RAID subsystem
	Input:			1. The flag indicates how to retrieve the event object(s), 
	                   this parameter can be one of the following values:
					   1a. FLAG_ALL, retrieve all event objects inside the RAID subsystem
					   1b. FLAG_NEW, retrieve only new event objects
	Output:			1. Number of event objects retrieved
	Return Values:	1. ARC_STATUS	
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcGetReqEventPage
	(
		IN	char bFlag, 	
		OUT int *iCount
	);

	/************************************************************************************
	Function Name:	ArcGetEventObject
	Function:		Get the individual event object
	Input:			1. Event object index 
	Output:			1. Pointer to an sEVENT_DATA to save the current event object
	Return Values:	1. boolean value, > 0 means successed
	Note:			1. 
	************************************************************************************/
	virtual BOOL ArcGetEventObject
	(
		IN	int nEvtNum, 
		OUT	pEVENT_DATA pEventObject
	);

	/************************************************************************************
	Function Name:	ArcClearEvent
	Function:		Clear all event buffer in the RAID subsystem
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcClearEvent
	( 
		void 
	);

	/************************************************************************************
	Function Name:	ArcCreateRaidSet
	Function:		Create RaidSet
	Input:			1. Device mask, bit0 -- drive 0, bit1 -- drive 1 ....
					   ex: Device mask = 0x0000000F means that create raidset with device 0,1,2 and 3
					2. RaidSet name ( <= 16 chars with null-terminated )
	Output:			None.	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateRaidSet
	(
		IN DWORD nDevMask, 
		IN char *szRaidName
	);
	

	/************************************************************************************
	Function Name:	ArcDelRaidSet
	Function:		Delete the particular RaidSet
	Input:			1. RaidSet number to specify which one to be deleted
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDelRaidSet
	(
		IN char nRaidNum
	);

	/************************************************************************************
	Function Name:	ArcExpandRaidSet
	Function:		Expand RaidSet with volumeset(s) modification
	Input:			1. RaidSet number to specify which one to be expanded
					2. Device mask(see ArcCreateRaidSet(...))
					3. The volumeset(s) will be modified 
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcExpandRaidSet
	( 
		IN	BYTE nRaidNum, 
		IN	DWORD nDevMask,
		IN	pVolModList	volmodlist = NULL
	);


	/************************************************************************************
	Function Name:	ArcActiveRaidSet
	Function:		Activate incomplete raid set
	Input:			1. RaidSet number to specify which one to be activated
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcActiveRaidSet
	(
		IN	BYTE nRaidNum
	);

	/************************************************************************************
	Function Name:	ArcCreateHotSpare
	Function:		Create Hot Spare Disk
	Input:			1. Device mask( see ArcCreateRaidSet(...))
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateHotSpare
	(
		IN DWORD nDevMask
	);

	/************************************************************************************
	Function Name:	ArcDeleteHotSpare
	Function:		Delete Hot Spare Disk
	Input:			1. Device mask(see ArcCreateRaidSet(...))
	Output:			None.		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDeleteHotSpare
	(
		IN DWORD nDevMask
	);	

	/************************************************************************************
	Function Name:	ArcCreateVolSet
	Function:		Create VolumeSet
	Input:			1. Pointer to a sVolumeSetCap structure that uses to create VolumeSet
	                2. Specifies the type to create over 2TB volumeset.
					   TYPE_64BIT_LBA: 64bitLBA(default), TYPE_4K_FOR_WIN: 4KByte for Windows
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateVolSet
	(
		IN pVolumeSetCap pVol,
		IN BYTE Over2TBType = TYPE_64BIT_LBA
	);

	/************************************************************************************
	Function Name:	ArcModifyVolSet
	Function:		Modify VolumeSet
	Input:			1. Pointer to a sNewVolumeSetCap structure that uses to modify VolumeSet
	Output:			None.  				
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcModifyVolSet
	(
		IN pNewVolumeSetCap pNewVol
	);

	/************************************************************************************
	Function Name:	ArcDelVolSet
	Function:		Delete VolumeSet
	Input:			1. VolumeSet number to specify which one to be deleted
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDelVolSet
	(
		IN BYTE nVolNum
	);

	/************************************************************************************
	Function Name:	ArcStartCheckVolume
	Function:		Start volume consistency check
	Input:			1. Volume number to specify which one be checked
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcStartCheckVolume
	(
		IN BYTE nVolNum
	);

	/************************************************************************************
	Function Name:	ArcStopCheckVolume
	Function:		Stop volume consistency check
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcStopCheckVolume
	(
		void
	);

	/************************************************************************************
	Function Name:	ArcLockEncryptedVolSet
	Function:		Make the AES key invalid(lock the encrypted volume)
	Input:			1. Volume number to specify which one be locked
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcLockEncryptedVolSet
	(
		IN BYTE nVolNum
	);

	/************************************************************************************
	Function Name:	ArcCreatePassthroughDisk
	Function:		Create pass-through disk
	Input:			1. Pointer to a sPassDiskCap structure that uses to create pass-through Disk
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreatePassthroughDisk
	(
		IN pPassDiskCap pDisk
	);

	/************************************************************************************
	Function Name:	ArcModifyPassthroughDisk
	Function:		Modify pass-through disk
	Input:			1. Pointer to a sPassDiskCap structure that uses to create pass-through Disk
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcModifyPassthroughDisk
	(
		IN pPassDiskCap pNewDiskCap
	);

	/************************************************************************************
	Function Name:	ArcDelPassthroughDisk
	Function:		Delete the pass-through disk
	Input:			1. Disk number specify which one to be deleted
	Output:			None.  				
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDelPassthroughDisk
	(
		IN BYTE nDrvNum
	);


	/************************************************************************************
    Function Name:	ArcIdentifyDisk
    Function:		Identify the disk
    Input:			1. Device mask(see ArcCreateRaidSet(...))
    Output:			None.  	
    Return Values:	1. ARC_STATUS
    Note:			1. Set the Device mask to zero to stop the disk identifying.
	                2. Password required if password enabled	                
	************************************************************************************/
    virtual ARC_STATUS ArcIdentifyDisk
    (
        IN DWORD nDriveMask
    );

	/************************************************************************************
	Function Name:	ArcResetController
	Function:		Restart controller
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
					2. After restart controller, it is essential to wait controller to 
					   get ready, the time may be 10~60 seconds depends on the models
	************************************************************************************/
	virtual ARC_STATUS ArcResetController( void );	
	
	/************************************************************************************
	Function Name:	ArcUpdateFirmware
	Function:		Update firmware
	Input:			1. The buffer contains the binary data of the firmware
					2. The actually size of firmware, not the buffer size
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:
	************************************************************************************/
	virtual ARC_STATUS ArcUpdateFirmware
	(
		IN char *fileBuf, 
		IN int fileSize
	);	
	
	/************************************************************************************
	Function Name:	ArcUpdateFirmware
	Function:		Update firmware
	Input:			1. The path that point to a firmware binary file				
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:
	************************************************************************************/
	virtual ARC_STATUS ArcUpdateFirmware
	(
		IN char *szFilePath
	);	

	/************************************************************************************
	Function Name:	ArcGetOnBoardNetInfo
	Function:		Get On-Board network information
	Input:			1. Pointer to a sNET_INFO structure that receives the information.
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:
	************************************************************************************/
	virtual ARC_STATUS ArcGetOnBoardNetInfo
	(
		OUT pNET_INFO pNetInfo
	);


	/************************************************************************************
	Function Name:	ArcGetSystemConfigInfo
	Function:		Get the system configurations
	Input:			None.
	Output:			1. Pointer to a sSYS_CONFIG_INFO structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcGetSystemConfigInfo
	(
		OUT pSYS_CONFIG_INFO pSysCfgInfo
	);


	/************************************************************************************
	Function Name:	ArcSetStaggerPowerOn
	Function:		Set stagger poweron mode
	Input:			1. Mode of the stagger poweron
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _STAGGER_POWERON
	{
		STAGGER_POWERON_0_4	= 0,	// 0.4
		STAGGER_POWERON_0_7,		// 0.7
		STAGGER_POWERON_1_0,		// 1.0
		STAGGER_POWERON_1_5,		// 1.5
		STAGGER_POWERON_2_0,		// 2.0
		STAGGER_POWERON_2_5,		// 2.5
		STAGGER_POWERON_3_0,		// 3.0
		STAGGER_POWERON_3_5,		// 3.5
		STAGGER_POWERON_4_0,		// 4.0
		STAGGER_POWERON_4_5,		// 4.5
		STAGGER_POWERON_5_0,		// 5.0
		STAGGER_POWERON_5_5,		// 5.5
		STAGGER_POWERON_6_0,		// 6.0
		TOTAL_STAGGER_POWERON
	}STAGGER_POWERON;

	virtual ARC_STATUS ArcSetStaggerPowerOn
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetDiskCache
	Function:		Set disk cache mode
	Input:			1. Mode of disk cache
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _HDD_DISK_CACHE
	{
		HDD_DISK_CACHE_AUTO = 0,
		HDD_DISK_CACHE_ENABLED,
		HDD_DISK_CACHE_DISABLED,
		TOTAL_HDD_DISK_CACHE
	}HDD_DISK_CACHE;

	virtual ARC_STATUS ArcSetDiskCache
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetHddReadAhead
	Function:		Set HDD read-ahead mode
	Input:			1. Mode of HDD read-ahead
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _HDD_READ_AHEAD
	{
		HDD_READ_AHEAD_ENABLED = 0,
		HDD_READ_AHEAD_DISABLED_MAXTOR,
		HDD_READ_AHEAD_DISABLED,
		TOTAL_HDD_READ_AHEAD
	}HDD_READ_AHEAD;

	virtual ARC_STATUS ArcSetHddReadAhead
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetTruncateMode
	Function:		Set truncate mode
	Input:			1. Mode of truncate
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _TRUNCATE_MODE
	{
		TRUNCATE_MODE_10G = 0,
		TRUNCATE_MODE_1G,
		TRUNCATE_MODE_DISABLED,
		TOTAL_TRUNCATE_MODE
	}TRUNCATE_MODE;

	virtual ARC_STATUS ArcSetTruncateMode
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetHTTPPort
	Function:		Set HTTP port
	Input:			1. HTTP port number
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetHTTPPort
	(
		IN WORD nPort
	);


	/************************************************************************************
	Function Name:	ArcSetTelnetPort
	Function:		Set telnet port
	Input:			1. Telnet port number
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetTelnetPort
	(
		IN WORD nPort
	);


	/************************************************************************************
	Function Name:	ArcSetSMTPPort
	Function:		Set SMTP port
	Input:			1. SMTP port number
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetSMTPPort
	(
		IN WORD nPort
	);

	/************************************************************************************
	Function Name:	ArcSetAutoActivate
	Function:		Set Auto-Activate
	Input:			1. Mode of auto-activate
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetAutoActivate
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetSMARTHddPoll
	Function:		Set SMART HDD Poll Mode
	Input:			1. Mode of SMART HDD poll
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetSMARTHddPoll
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetSMARTOptionForHdd
	Function:		Set SMART Option For HDD
	Input:			1. Mode of SMART Option For HDD
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _SMART_OPTION_FOR_HDD
	{		
		SMART_OPTION_FAILED_DRIVE,
		SMART_OPTION_FAILED_DRIVE_IF_HOTSPARE_EXIST,
		SMART_OPTION_ALERT,
		TOTAL_SMART_OPTION_FOR_HDD
	}SMART_OPTION_FOR_HDD;

	virtual ARC_STATUS ArcSetSMARTOptionForHdd
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetSMARTPollingInterval
	Function:		Set SMART HDD Polling Interval
	Input:			1. Mode of SMART HDD Polling Interval
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _SMART_POLLING_INTERVAL
	{
		TIME_SMART_POLLING_ONDEMAND,
		TIME_SMART_POLLING_30_SECS,
		TIME_SMART_POLLING_1_MIN,
		TIME_SMART_POLLING_5_MINS,
		TIME_SMART_POLLING_10_MINS,
		TIME_SMART_POLLING_20_MINS,
		TIME_SMART_POLLING_30_MINS,
		TIME_SMART_POLLING_1_HOUR,
		TIME_SMART_POLLING_2_HOURS,
		TIME_SMART_POLLING_3_HOURS,
		TIME_SMART_POLLING_6_HOURS,
		TIME_SMART_POLLING_12_HOURS,
		TOTAL_SMART_POLLING_INTERVAL
	}SMART_POLLING_INTERVAL;

	virtual ARC_STATUS ArcSetSMARTPollingInterval
	(
		IN BYTE nMode
	);
	
	/************************************************************************************
	Function Name:	ArcSetHddSpinDownMode
	Function:		Set HDD spin down Mode
	Input:			1. Mode of HDD spin down
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _TIME_TO_SPIN_DOWN_IDLE_HDD
	{
		TIME_HDD_SPIN_DOWN_DISABLED,
		TIME_HDD_SPIN_DOWN_1,
		TIME_HDD_SPIN_DOWN_3,
		TIME_HDD_SPIN_DOWN_5,
		TIME_HDD_SPIN_DOWN_10,
		TIME_HDD_SPIN_DOWN_15,
		TIME_HDD_SPIN_DOWN_20,
		TIME_HDD_SPIN_DOWN_30,
		TIME_HDD_SPIN_DOWN_40,
		TIME_HDD_SPIN_DOWN_60,
		TOTAL_TIME_HDD_SPIN_DOWN
	}TIME_TO_SPIN_DOWN_IDLE_HDD;

	virtual ARC_STATUS ArcSetHddSpinDownMode
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetHddLowPowerIdleMode
	Function:		Set HDD Low Power Idle Mode
	Input:			1. Mode of HDD low power idle
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _TIME_TO_HDD_LOW_POWER_IDLE
	{
		TIME_HDD_LOW_POWER_DISABLED,
		TIME_HDD_LOW_POWER_2,
		TIME_HDD_LOW_POWER_3,
		TIME_HDD_LOW_POWER_4,
		TIME_HDD_LOW_POWER_5,
		TIME_HDD_LOW_POWER_6,
		TIME_HDD_LOW_POWER_7,
		TOTAL_TIME_HDD_LOW_POWER
	}TIME_TO_HDD_LOW_POWER_IDLE;

	virtual ARC_STATUS ArcSetHddLowPowerIdleMode
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetHddLowRPMMode
	Function:		Set HDD Low RPM Mode 
	Input:			1. Mode of hdd low RPM
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	typedef enum _TIME_TO_HDD_LOW_RPM_MODE
	{
		TIME_HDD_LOW_RPM_DISABLED,
		TIME_HDD_LOW_RPM_10,
		TIME_HDD_LOW_RPM_20,
		TIME_HDD_LOW_RPM_30,
		TIME_HDD_LOW_RPM_40,
		TIME_HDD_LOW_RPM_50,
		TIME_HDD_LOW_RPM_60,
		TOTAL_TIME_HDD_LOW_RPM
	}TIME_TO_HDD_LOW_RPM_MODE;

	virtual ARC_STATUS ArcSetHddLowRPMMode
	(
		IN BYTE nMode
	);


	/************************************************************************************
	Function Name:	ArcSetEmptyHddSlotLed
	Function:		Set empty HDD slot LED mode
	Input:			1. Mode of empty HDD slot LED(0: ON, 1:OFF)
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetEmptyHddSlotLed
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetCpuFanDetect
	Function:		Set CPU fan detection mode
	Input:			1. Mode of CPU detection mode
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetCpuFanDetection
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	IsEnclosure
	Function:		
	Input:			None
	Output:			None
	Return Values:	'true' if the target subsystem is an Enclosure.
	Note:
	************************************************************************************/
	BOOL IsEnclosure();


	/************************************************************************************
	Function Name:	ArcGetEnclosureProperty
	Function:		Get the Enclosure property
	Input:			1. encnum: Enclosure#	                
	Output:			1. Pointer to a sGUI_ENC_PROPERTY structure that receives the information.
	Return Values:	ARC_STATUS
	Note:
	************************************************************************************/
	virtual ARC_STATUS ArcGetEnclosureProperty
	(
		IN BYTE nEncnum, 
		OUT pGUI_ENC_PROPERTY pEncProperty
	);

	/************************************************************************************
	Function Name:	ArcGetEnclosureDescriptor
	Function:		Get the Enclosure descriptor
	Input:			1. nEncnum: Enclosure#
	                2. nItem: see 'enum _SAS_HW' 
					3. nBufsize: the size of output buffer
	Output:			1. pDescbuf: Pointer to a buffer that receives the item descriptor.
	Return Values:	ARC_STATUS
	Note:
	************************************************************************************/	
	virtual ARC_STATUS ArcGetEnclosureDescriptor
	(
		IN BYTE nEncnum, 
		IN BYTE nItem, 
		OUT BYTE *pDescbuf, 
		IN int nBufsize
	);


	/************************************************************************************
	Function Name:	ArcGetEnclosureHwMonitor
	Function:		Get the Enclosure H/W monitor
	Input:			1. nEncnum: Enclosure#	                
	Output:			1. Pointer to a sSENSORS structure that receives the information.
	Return Values:	ARC_STATUS
	Note:
	************************************************************************************/	
	virtual ARC_STATUS ArcGetEnclosureHwMonitor
	(
		IN BYTE nEncnum, 
		OUT pSENSORS pHwInfo
	);

	/************************************************************************************
	Function Name:	ArcGetPhysicalDriveInfo
	Function:		Get the physical drive information
	Input:			1. nEncnum: Enclosure#
					2. nDrvnum: Drive# in Enclsoure#
	Output:			1. Pointer to a sGUI_PHY_DRV structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			
	************************************************************************************/
	virtual ARC_STATUS ArcGetPhysicalDriveInfo
	(		
		IN BYTE nEncnum, 
		IN BYTE nDrvnum, 
		OUT	pGUI_PHY_DRV pPhyDrv	
	);

	/************************************************************************************
	Function Name:	ArcCreateRaidSet
	Function:		Create RaidSet
	Input:			1. pDevPairList: Pointer to a sDEVICE_LIST structure containing a list
					   of devices
				    2. RaidSet name ( <= 16 chars with null-terminated )
					3. nMode: 1 for 16 Volumesets, 0 for 128 Volumesets(default)
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateRaidSet
	(
		IN pDEVICE_LIST pDevPairList,
		IN char *szRaidName,
		IN BYTE nMode = 0
	);

	
	/************************************************************************************
	Function Name:	ArcExpandRaidSet
	Function:		Expand RaidSet With Volumeset(s) Modification
	Input:			1. RaidSet number to specify which one to be expanded
					2. pDevPairList: Pointer to a sDEVICE_LIST structure containing a list
					   of devices					
					3. The volumeset(s) that will be modified 
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcExpandRaidSet
	( 
		IN BYTE nRaidNum, 
		IN pDEVICE_LIST pDevPairList,
		IN pVolModList	volmodlist = NULL
	);

	/************************************************************************************
	Function Name:	ArcCreateHotSpare
	Function:		Create Hot Spare Disk
	Input:			1. pDevPairList: Pointer to a sDEVICE_LIST structure containing a list
					   of devices			
					2. Type of hotspare disk
					3. RaidSet# for hotspare type == HOT_SPARE_LOCAL_RAID
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/	
	virtual ARC_STATUS ArcCreateHotSpare
	(
		IN pDEVICE_LIST pDevPairList,
		IN BYTE nType = HOT_SPARE_GLOBAL,
		IN WORD nRaidnum = 0
	);

	/************************************************************************************
	Function Name:	ArcDeleteHotSpare
	Function:		Delete Hot Spare Disk
	Input:			1. pDevPairList: Pointer to a sDEVICE_LIST structure containing a list
					   of devices	
	Output:			None.		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDeleteHotSpare
	(
		IN pDEVICE_LIST pDevPairList
	);
	
	/************************************************************************************
	Function Name:	ArcCreatePassthroughDisk
	Function:		Create Pass-Through Disk
	Input:			1. nEncnum: Enclosure#
					2. nDrvnum: Drive# in Enclsoure#
					3. pDisk: Pointer to a sPassDiskCap structure that uses to create pass-through Disk
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. This function use the nDrvnum instead of pDisk->nDevNum
					2. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreatePassthroughDisk
	(
		IN BYTE nEncnum, 
		IN BYTE nDrvnum,
		IN pPassDiskCap pDisk
	);

	/************************************************************************************
	Function Name:	ArcModifyPassthroughDisk
	Function:		Modify Pass-Through Disk
	Input:			1. nEncnum: Enclosure#
					2. nDrvnum: Drive# in Enclsoure#
					3. pNewDiskCap: Pointer to a sPassDiskCap structure that uses to create pass-through Disk
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. This function use the nDrvnum instead of pNewDiskCap->nDevNum
					2. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcModifyPassthroughDisk
	(
		IN BYTE nEncnum, 
		IN BYTE nDrvnum, 
		IN pPassDiskCap pNewDiskCap
	);

	/************************************************************************************
	Function Name:	ArcDelPassthroughDisk
	Function:		Delete Pass-Through Disk
	Input:			1. nEncnum: Enclosure#
					2. nDrvnum: Drive# in Enclsoure#
	Output:			None.  				
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDelPassthroughDisk
	(
		IN BYTE nEncnum, 
		IN BYTE nDrvnum
	);


	/************************************************************************************
    Function Name:	ArcSASIdentifyDisk
    Function:		Identify the disk
    Input:			1. nTarget: The target to be identified.
	                2. nEncnum: Enclosure#
					3. nDrvnum: Drive# in Enclsoure#
    Output:			None.  	
    Return Values:	1. ARC_STATUS
    Note:			
	                
	************************************************************************************/


	enum _IDENTIFY_TARGET
	{
		TARGET_DRIVE = 0,	// the target is disk
		TARGET_ENCLOSURE,	// the target is enclosure
		TARGET_OFF,			// end identifying
		TARGET_ALL
	};
    virtual ARC_STATUS ArcSASIdentifyDisk
    (
		IN BYTE	nTarget,
        IN BYTE nEncnum = 0xFF, 
		IN BYTE nDrvnum = 0xFF
    );	

	

	/************************************************************************************
	Function Name:	ArcCreateVolSetR5060
	Function:		Create Raid30/50/60 VolumeSet
	Input:			1. nRaidNumber: Number of RaidSets use to create Raid30/50/60
	                   VolumeSet
	                2. nRaidArray: Pointer to an array containing index of RaidSets
	                3. Pointer to a sVolumeSetCap structure that uses to create VolumeSet
	                4. Specifies the type to create over 2TB volumeset.
					   TYPE_64BIT_LBA: 64bitLBA(default), TYPE_4K_FOR_WIN: 4KByte for Windows
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateVolSetR5060
	(
		IN BYTE nRaidNumber,
		IN BYTE nRaidArray[],
		IN pVolumeSetCap pVol,
		IN BYTE Over2TBType = TYPE_64BIT_LBA
	);


//////////////////////////////////////////
//////    MAIL   Functions    ////////////
//////////////////////////////////////////

	/************************************************************************************
	Function Name:	ArcGetMailInfo
	Function:		Get the mail notification information
	Input:			None.
	Output:			1. Pointer to a sMAIL_INFO structure that receives the information.
	Return Values:	ARC_STATUS
	Note:
	************************************************************************************/	
	virtual ARC_STATUS ArcGetMailInfo
	(		
		OUT pMAIL_INFO pMailInfo
	);

	virtual ARC_STATUS	ArcSetMailSmtpIp
	(
		IN char *szSmtpIp	// format: string with null-terminated "192.168.0.1"
	);

	virtual ARC_STATUS	ArcSetMailSmtpLoginAccount
	(
		IN char *szName,			// SMTP server login name
		IN char *szPassword			// SMTP server login password
	);

	enum _MAIL_ADDR_ITEM
	{
		MAIL_ITEM_SENDER,			
		MAIL_ITEM_MAILTO1,			
		MAIL_ITEM_MAILTO2,
		MAIL_ITEM_MAILTO3,
		MAIL_ITEM_MAILTO4,
		TOTAL_MAIL_ITEM
	};

	virtual ARC_STATUS	ArcSetMailAddr
	(
		IN DWORD	  nItem,	// enum _MAIL_ADDR_ITEM
		IN pMAIL_ADDR pMail
	);

	virtual ARC_STATUS	ArcSetMailEvtLevel
	(
		IN DWORD	nEvtLevel,	// enum _EVT_LEVEL
		IN DWORD    nItem = MAIL_ITEM_MAILTO1 // enum _MAIL_ADDR_ITEM
	);

	virtual ARC_STATUS	ArcSetMailNoEvtFor24Hours
	(
		IN DWORD	nNotificationForNoEvent,	// default enabled
		IN DWORD    nItem = MAIL_ITEM_MAILTO1 // enum _MAIL_ADDR_ITEM
	);



//////////////////////////////////////////////
//////////    SNMP   Functions    ////////////
//////////////////////////////////////////////

	/************************************************************************************
	Function Name:	ArcGetSnmpInfo
	Function:		Get the SNMP notification information
	Input:			None.
	Output:			1. Pointer to a sSNMP_CONFIG structure that receives the information.
	Return Values:	ARC_STATUS
	Note:
	************************************************************************************/	
	virtual ARC_STATUS ArcGetSnmpInfo
	(		
		OUT pSNMP_CONFIG pSnmpInfo
	);


	enum _SNMP_TRAP_IP_ITEM
	{
		SNMP_TRAP_IP1,
		SNMP_TRAP_IP2,
		SNMP_TRAP_IP3,
		TOTAL_SNMP_TRAP_IP
	};

	virtual ARC_STATUS ArcSetSnmpTrapIp
	(
		IN DWORD	nItem,	// enum _SNMP_TRAP_IP_ITEM
		char		*szIp
	);
	


	enum _SNMP_TRAP_PORT_ITEM
	{
		SNMP_TRAP_PORT1,
		SNMP_TRAP_PORT2,
		SNMP_TRAP_PORT3,
		TOTAL_SNMP_TRAP_PORT
	};

	virtual ARC_STATUS ArcSetSnmpTrapPort
	(
	    IN DWORD nItem,	// enum _SNMP_TRAP_PORT_ITEM
		WORD	port
	);

	virtual ARC_STATUS ArcSetSnmpCommunity
	(
		IN char	*szCommunity	// SNMP community string with null-terminated
	);

	virtual ARC_STATUS ArcSetSnmpContact
	(
		IN char	*szContact		// contact string with null-terminated
	);

	virtual ARC_STATUS ArcSetSnmpName
	(
		IN char	*szName			// name string with null-terminated
	);

	virtual ARC_STATUS ArcSetSnmpLocation
	(
		IN char	*szLocation		// location string with null-terminated
	);

	virtual ARC_STATUS	ArcSetSnmpEvtLevel
	(
		DWORD	nEvtLevel	// enum _EVT_LEVEL
	);


//////////////////////////////////////////////
//////////    NTP   Functions    ////////////
//////////////////////////////////////////////


	/************************************************************************************
	Function Name:	ArcSetTimeZone
	Function:		Set timezone
	Input:			1. Mode of timezone ( see below )               
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	/*  0   (GMT-12:00)International Date Line West */                          
	/*  1   (GMT-11:00)Midway Island, Samoa */                                  
	/*  2   (GMT-10:00)Hawaii */                                                
	/*  3   (GMT-09:00)Alaska */                                                
	/*  4   (GMT-08:00)Pacific Time(US&Canada), Tijuana */                      
	/*  5   (GMT-07:00)Arizona */                                               
	/*  6   (GMT-07:00)Chihuahua, La Paz, Mazatlan */                           
	/*  7   (GMT-07:00)Mountain Time(US&Canada) */                              
	/*  8   (GMT-06:00)Central America */                                       
	/*  9   (GMT-06:00)Central Time(US&Canada) */                               
	/* 10   (GMT-06:00)Guadalajara, Mexico City, Monterrey */                   
	/* 11   (GMT-06:00)Saskatchewan */                                          
	/* 12   (GMT-05:00)Bogota, Lima, Quito */                                   
	/* 13   (GMT-05:00)Eastern Time(US&Canada) */                               
	/* 14   (GMT-05:00)Indiana(East) */                                         
	/* 15   (GMT-04:00)Atlantic Time(Canada) */                                 
	/* 16   (GMT-04:00)Caracas, La Paz */                                       
	/* 17   (GMT-04:00)Santiago */                                              
	/* 18   (GMT-03:30)Newfoundland */                                          
	/* 19   (GMT-03:00)Brasilia */                                              
	/* 20   (GMT-03:00)Buenos Aires, Georgetown */                              
	/* 21   (GMT-03:00)Greenland */                                             
	/* 22   (GMT-02:00)Mid-Atlantic */                                          
	/* 23   (GMT-01:00)Azores */                                                
	/* 24   (GMT-01:00)Cape Verde Is. */                                        
	/* 25   (GMT+00:00)Casablanca, Monrovia */                                  
	/* 26   (GMT+00:00)Greenwich Mean Time, Dublin, Edinburgh, Lisbon, London */
	/* 27   (GMT+01:00)Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna */      
	/* 28   (GMT+01:00)Belgrade, Bratislava, Budapest, Ljubljana, Prague */     
	/* 29   (GMT+01:00)Brussels, Copenhagen, Madrid, Paris */                   
	/* 30   (GMT+01:00)Sarajevo, Skopje, Warsaw, Zagreb */                      
	/* 31   (GMT+01:00)West Central Africa */                                   
	/* 32   (GMT+02:00)Athens, Beirut, Istanbul, Minsk */                       
	/* 33   (GMT+02:00)Bucharest */                                             
	/* 34   (GMT+02:00)Cairo */                                                 
	/* 35   (GMT+02:00)Harare, Pretoria */                                      
	/* 36   (GMT+02:00)Helsinki, Kyiv, Riga, Sofia, Tallinn, Vilnius */         
	/* 37   (GMT+02:00)Jerusalem */                                             
	/* 38   (GMT+03:00)Baghdad */                                               
	/* 39   (GMT+03:00)Kuwait, Riyadh */                                        
	/* 40   (GMT+03:00)Moscow, St. Petersburg, Volgograd */                     
	/* 41   (GMT+03:00)Nairobi */                                               
	/* 42   (GMT+03:30)Tehran */                                                
	/* 43   (GMT+04:00)Abu Dhabi, Muscat */                                     
	/* 44   (GMT+04:00)Baku, Tbilisi, Yerevan */                                
	/* 45   (GMT+04:30)Kabul */                                                 
	/* 46   (GMT+05:00)Ekaterinburg */                                          
	/* 47   (GMT+05:00)Islamabad, Karachi, Tashkent */                          
	/* 48   (GMT+05:30)Chennai, Kolkata, Mumbai, New Delhi */                   
	/* 49   (GMT+05:45)Kathmandu */                                             
	/* 50   (GMT+06:00)Almaty, Novosibirsk */                                   
	/* 51   (GMT+06:00)Astana, Dhaka */                                         
	/* 52   (GMT+06:00)Sri Jayawardenepura */                                   
	/* 53   (GMT+06:30)Rangoon */                                               
	/* 54   (GMT+07:00)Bangkok, Hanoi, Jakarta */                               
	/* 55   (GMT+07:00)Krasnoyarsk */                                           
	/* 56   (GMT+08:00)Beijing, Chongging, Hong Kong, Urumqi */                 
	/* 57   (GMT+08:00)Irkutsk, Ulaan Bataar */                                 
	/* 58   (GMT+08:00)Kuala Lumpur, Singapore */                               
	/* 59   (GMT+08:00)Perth */                                                 
	/* 60   (GMT+08:00)Taipei */                                                
	/* 61   (GMT+09:00)Osaka, Sapporo, Tokyo */                                 
	/* 62   (GMT+09:00)Seoul */                                                 
	/* 63   (GMT+09:00)Yakutsk */                                               
	/* 64   (GMT+09:30)Adelaide */                                              
	/* 65   (GMT+09:30)Darwin */                                                
	/* 66   (GMT+10:00)Brisbane */                                              
	/* 67   (GMT+10:00)Canberra, Melbourne, Sydney */                           
	/* 68   (GMT+10:00)Guam, Port Moresby */                                    
	/* 69   (GMT+10:00)Hobart */                                                
	/* 70   (GMT+10:00)Vladivostok */                                           
	/* 71   (GMT+11:00)Magadan, Solomon Is., New Caledonia */                   
	/* 72   (GMT+12:00)Auckland, Wellington */                                  
	/* 73   (GMT+12:00)Fiji, Kamchatka, Marshall Is. */                         
	/* 74   (GMT+13:00)Nuku'alofa */                                            

	virtual ARC_STATUS ArcSetTimeZone
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetDayLightSaving
	Function:		Set day light saving
	Input:			1. Mode day light saving        
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	virtual ARC_STATUS ArcSetDayLightSaving
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetNTPIp
	Function:		Set NTP IP
	Input:			1. Null-terminated character string representing a number expressed in the Internet standard "."
					   (dotted) notation, format: "xxx.xxx.xxx.xxx" ex: 192.168.001.100
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/

	enum _NTP_IP_ITEM
	{
		NTP_IP1,
		NTP_IP2,		
		TOTAL_NTP_IP
	};

	virtual ARC_STATUS ArcSetNtpIp
	(
		IN DWORD	nItem,	// enum _NTP_IP_ITEM
		IN char *szNTPIp
	);

	/************************************************************************************
	Function Name:	ArcGetNtpInfo
	Function:		Get the NTP information
	Input:			None.
	Output:			1. Pointer to a pNTP_CONFIG structure that receives the information.
	Return Values:	ARC_STATUS
	Note:
	************************************************************************************/	
	virtual ARC_STATUS ArcGetNtpInfo
	(
		OUT pNTP_CONFIG pNtpInfo
	);

	/************************************************************************************
	Function Name:	ArcSetGatewayIp
	Function:		Set Gateway IP
	Input:			1. Null-terminated character string representing a number expressed in the Internet standard "."
					   (dotted) notation, format: "xxx.xxx.xxx.xxx" ex: 192.168.001.100
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS	ArcSetGatewayIp
	(
		IN char *szGatewayIp
	);
	
	/************************************************************************************
	Function Name:	ArcOfflineRaidSet
	Function:		Offline the particular RaidSet
	Input:			1. RaidSet number
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcOfflineRaidSet
	(
		IN char nRaidNum
	);	

	/************************************************************************************
	Function Name:	ArcFailDisk
	Function:		Make one disk failed
	Input:			1. nEncnum: Enclosure#
  					2. nDrvnum: Drive# in Enclsoure#
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcFailDisk
	(
		IN BYTE nEncnum, 
		IN BYTE nDrvnum
	);

	/************************************************************************************
	Function Name:	ArcSetReadAhead
	Function:		Set Controller ReadAhead Buffer Size
	Input:			1. Mode of controller read-ahead
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _READAHEAD_BUFFER
	{
		READAHEAD_BUFFER_AUTO = 0,
		READAHEAD_BUFFER_512K,
		READAHEAD_BUFFER_1MB,
		READAHEAD_BUFFER_2MB,
		READAHEAD_BUFFER_4MB,
		READAHEAD_BUFFER_6MB,
		READAHEAD_BUFFER_8MB,
		READAHEAD_BUFFER_10MB,
		READAHEAD_BUFFER_12MB,
		READAHEAD_BUFFER_14MB,
		READAHEAD_BUFFER_16MB,
		TOTAL_READAHEAD_BUFFER
	}READAHEAD_BUFFER;

	virtual ARC_STATUS ArcSetReadAhead
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetTLER
	Function:		Set Time-Limited Error Recovery
	Input:			1. Mode of TLER
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _TLER
	{
		TLER_DEFAULT = 0,
		TLER_DISABLED,
		TLER_5_SECONDS,
		TLER_6_SECONDS,
		TLER_7_SECONDS,
		TOTAL_TLER
	}TLER;

	virtual ARC_STATUS ArcSetTLER
	(
		IN BYTE nMode
	);
	
	/************************************************************************************
	Function Name:	ArcSetBufferThreshold
	Function:		Indicating how much data should be kept in resident cache memory 
	                (how full cache should get) before controller starts to flush data 
					onto the hard drives
	Input:			1. Mode of buffer threshold
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _BUFFER_THRESHOLD
	{
		BUFFER_THRESHOLD_75 = 0,	// 75%
		BUFFER_THRESHOLD_50,		// 50%
		BUFFER_THRESHOLD_25,		// 25%
		BUFFER_THRESHOLD_5,			// 5%
		TOTAL_BUFFER_THRESHOLD
	}BUFFER_THRESHOLD;

	virtual ARC_STATUS ArcSetBufferThreshold
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetTimeout
	Function:		Defines the time that RAID controller will wait for a hard disk to 
	                respond to a command
	Input:			1. Mode of timeout
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _TIMEOUT
	{
		TIMEOUT_8_SECONDS = 0,
		TIMEOUT_7_SECONDS,
		TIMEOUT_6_SECONDS,
		TIMEOUT_5_SECONDS,
		TIMEOUT_4_SECONDS,
		TIMEOUT_3_SECONDS,
		TIMEOUT_12_SECONDS,
		TIMEOUT_17_SECONDS,
		TIMEOUT_22_SECONDS,
		TIMEOUT_27_SECONDS,
		TIMEOUT_32_SECONDS,
		TOTAL_TIMEOUT
	}TIMEOUT;

	virtual ARC_STATUS ArcSetTimeout
	(
		IN BYTE nMode
	);

	 
	/************************************************************************************
	Function Name:	ArcSetRetryCount
	Function:		Defines the number of access that will be attempted before the 
	                current command from the RAID controller to the disk drive is aborted	                
	Input:			1. Mode of retry count
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _RETRY_COUNT
	{
		RETRY_2_TIMES = 0,
		RETRY_3_TIMES,
		TOTAL_RETRY_COUNT
	}RETRY_COUNT;

	virtual ARC_STATUS ArcSetRetryCount
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetAVStream
	Function:		RAID controllers are required to have not only the function of processing
                    ordinary data but also the function of dealing with AV (audio/video) 
					stream data needing real-time processing. Since the bus cycle used 
					in RAID controller was designed to transfer the computer data exactly, 
					it was unsuitable for the transfer of AV stream needing great band widths. 
					They are required to do some setting for the handshaking during the 
					processing of stream data. This setting is an object of transferring 
					stream data efficiently on an existing RAID controller. Normally you 
					should not need to modify this value.	                
	Input:			1. Mode of AV stream
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _AV_STREAM
	{
		AV_STREAM_6 = 0,
		AV_STREAM_11,
		AV_STREAM_16,
		AV_STREAM_21,
		AV_STREAM_26,
		AV_STREAM_31,
		AV_STREAM_36,
		AV_STREAM_56,
		AV_STREAM_76,
		AV_STREAM_96,
		AV_STREAM_116,
		AV_STREAM_136,
		AV_STREAM_156,
		AV_STREAM_176,
		AV_STREAM_196,
		AV_STREAM_216,
		AV_STREAM_236,
		AV_STREAM_256,
		TOTAL_AV_STREAM
	}AV_STREAM;

	virtual ARC_STATUS ArcSetAVStream
	(
		IN BYTE nMode
	);


	/************************************************************************************
	Function Name:	ArcSetOptimizeAVRecording
	Function:		AV recording option is for video recording (no time limit), but if
                    used in normal operation, performance may be degraded	                
	Input:			1. Mode of optimize AV recording
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _OPTIMIZE_AV_RECODING
	{
		OPTIMIZE_AV_DISABLED = 0,
		OPTIMIZE_AV_MODE_1,
		OPTIMIZE_AV_MODE_2,
		OPTIMIZE_AV_MODE_3,
		TOTAL_OPTIMIZE_AV_RECODING
	}OPTIMIZE_AV_RECODING;

	virtual ARC_STATUS ArcSetOptimizeAVRecording
	(
		IN BYTE nMode
	);


	/************************************************************************************
	Function Name:	ArcSetOptimizeAVEdition
	Function:		HD edit option is not for long time continuous AV recording (may
                    lose data if power loss). It is recommended for hours of operation
                    and has some interval idling for house keeping action	                
	Input:			1. Mode of optimize AV edition
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _OPTIMIZE_HD_EDITION
	{
		OPTIMIZE_HD_EDITION_DISABLED = 0,
		OPTIMIZE_HD_EDITION_MODE_1,
		OPTIMIZE_HD_EDITION_MODE_2,
		OPTIMIZE_HD_EDITION_MODE_3,
		OPTIMIZE_HD_EDITION_MODE_WIN,
		TOTAL_OPTIMIZE_HD_EDITION
	}OPTIMIZE_HD_EDITION;

	virtual ARC_STATUS ArcSetOptimizeAVEdition
	(
		IN BYTE nMode
	);


	/************************************************************************************
	Function Name:	ArcGetAdvancedFeaturesInfo
	Function:		Get system advanced features information
	Input:			None.
	Output:			1. Pointer to a sSYS_ADVANCED_FEATURES_INFO structure that receives the information.
	Return Values:	ARC_STATUS
	Note:
	************************************************************************************/	
	typedef struct _SYS_ADVANCED_FEATURES_INFO
	{
		BYTE	readahead;
		BYTE	tler;
		BYTE	bufferthreshold;
		BYTE	timeout;
		BYTE	retry;
		BYTE	avstreams;
		BYTE	optimizeav;
		BYTE	optimizehdedit;
		BYTE    phystatus;
		BYTE    readperformancemargin;
		BYTE    writeperformancemargin;
		BYTE    readanddiscardparitydata;
		BYTE    hitachisataspeed;
		BYTE    wdcsataspeed;
		BYTE    seagatesataspeed;
		BYTE    reserved[1];
	}sSYS_ADVANCED_FEATURES_INFO, *pSYS_ADVANCED_FEATURES_INFO;
	
	virtual ARC_STATUS ArcGetAdvancedFeaturesInfo
	(
		pSYS_ADVANCED_FEATURES_INFO pAdvanceInfo
	);


	/************************************************************************************
	Function Name:	ArcSetVolumeDataReadAhead
	Function:		Set volumeset data read-ahead                
	Input:			1. Mode of volume data read-ahead
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _VOL_DATA_READ_AHEAD
	{
		VOL_READ_AHEAD_NORMAL = 0,
		VOL_READ_AHEAD_AGGRESSIVE,
		VOL_READ_AHEAD_CONSERVATIVE,
		VOL_READ_AHEAD_DISABLED,
		TOTAL_VOL_READ_AHEAD
	}VOL_DATA_READ_AHEAD;

	virtual ARC_STATUS ArcSetVolumeDataReadAhead
	(
		IN BYTE nMode
	);


	/************************************************************************************
	Function Name:	ArcSetHddQueueDepth
	Function:		Set hdd queue depth
	Input:			1. Mode of hdd queue depth
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _HDD_QUEUE_DEPTH
	{
		HDD_QUEUE_DEPATH_1 = 0,
		HDD_QUEUE_DEPATH_2,
		HDD_QUEUE_DEPATH_4,
		HDD_QUEUE_DEPATH_8,
		HDD_QUEUE_DEPATH_16,
		HDD_QUEUE_DEPATH_32,
		TOTAL_HDD_QUEUE_DEPATH
	}HDD_QUEUE_DEPTH;

	virtual ARC_STATUS ArcSetHddQueueDepth
	(
		IN BYTE nMode
	);


	/************************************************************************************
	Function Name:	ArcSetSataNCQ
	Function:		Set SATA NCQ support
	Input:			1. Mode of NCQ
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _NCQ_SUPPORT
	{
		NCQ_ENABLED = 0,
		NCQ_DISABLED,
		TOTAL_NCQ
	}NCQ_SUPPORT;

	virtual ARC_STATUS ArcSetSataNCQ
	(
		IN BYTE nMode
	);

	/************************************************************************************
	Function Name:	ArcSetSASMux
	Function:		Set SAS Mux
	Input:			1. nMode: Mode of SAS Mux	                
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _SAS_MUX_SETTING
	{
		SAS_MUX_AUTO = 0,
		SAS_MUX_INTERNAL,
		SAS_MUX_EXTERNAL,
		TOTAL_SAS_MUX
	}SAS_MUX_SETTING;

	virtual ARC_STATUS ArcSetSASMux
	(
		IN BYTE nMode		
	);	

	/************************************************************************************
	Function Name:	ArcSetSES2Support
	Function:		Set SES2 Support
	Input:			1. nMode: Mode of SES2 Support
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _SES2_SUPPORT
	{
		SES2_ENABLE = 0,
		SES2_DISABLE,
		TOTAL_SES2_SUPPORT,
	}SES2_SUPPORT;

	virtual ARC_STATUS ArcSetSES2Support
	(
		IN BYTE nMode		
	);	


	/************************************************************************************
	Function Name:	ArcSetSATA150Support
	Function:		Set SATA150 Support
	Input:			1. nMode: Mode of SATA150 Support
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _SATA150_SUPPORT
	{
		SATA150_ENABLE = 0,
		SATA150_DISABLE,
		TOTAL_SATA150_SUPPORT,
	}SATA150_SUPPORT;

	virtual ARC_STATUS ArcSetSATA150Support
	(
		IN BYTE nMode		
	);	


	/************************************************************************************
	Function Name:	ArcSetMaxCmdLen
	Function:		Set Max Command Length Support
	Input:			1. nMode: Mode of Max Command Length
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _MAX_CMD_LEN
	{
		MAX_CMD_LEN_148K = 0,
		MAX_CMD_LEN_256K,
		MAX_CMD_LEN_512K,
		MAX_CMD_LEN_1M,
		MAX_CMD_LEN_2M,
		MAX_CMD_LEN_4M,
		TOTAL_MAX_CMD_LEN,
	}MAX_CMD_LEN;

	virtual ARC_STATUS ArcSetMaxCmdLen
	(
		IN BYTE nMode		
	);	

	/************************************************************************************
	Function Name:	ArcSetReadPerformanceMargin
	Function:		Set Read Performance Margin
	Input:			1. nMode: Mode of Read Performance Margin,from value 0(0%) ~ value 30(30%)
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/		
	virtual ARC_STATUS ArcSetReadPerformanceMargin
	(
		IN BYTE nMode		
	);	

	/************************************************************************************
	Function Name:	ArcSetWritePerformanceMargin
	Function:		Set Write Performance Margin
	Input:			1. nMode: Mode of Write Performance Margin,from value 0(0%) ~ value 30(30%)
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/		
	virtual ARC_STATUS ArcSetWritePerformanceMargin
	(
		IN BYTE nMode		
	);	

	/************************************************************************************
	Function Name:	ArcSetReadAndDiscardParityData
	Function:		Set Read And Discard Parity Data
	Input:			1. nMode: Mode of Read And Discard Parity Data
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _RDPD_SUPPORT
	{
		RDPD_DISABLE = 0,
		RDPD_ENABLE,
		TOTAL_RDPD_SUPPORT,
	}RDPD_SUPPORT;

	virtual ARC_STATUS ArcSetReadAndDiscardParityData
	(
		IN BYTE nMode		
	);	
	
	/************************************************************************************
	Function Name:	ArcSetSataHddSpeed
	Function:		Set WDC Sata Hdd Speed
	Input:			1. nVendor: The vendor of HDD
	                2. nMode: Mode of Sata Hdd Speed
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Supports WDC, Hitachi and Seagate.
	************************************************************************************/		
	typedef enum _SATA_HDD_SPEED
	{
		SATA_HDD_SPEED_DEFAULT = 0,
		SATA_HDD_SPEED_1_5G,
		SATA_HDD_SPEED_3_0G,
		SATA_HDD_SPEED_6_0G,
		TOTAL_SATA_HDD_SPEED
	}SATA_HDD_SPEED;

	typedef enum _SATA_HDD_VENDOR
	{
		SATA_HDD_VENDOR_WDC = 0,
		SATA_HDD_VENDOR_HITACHI,
		SATA_HDD_VENDOR_SEAGATE,		
		TOTAL_SATA_HDD_VENDOR
	}SATA_HDD_VENDOR;

	virtual ARC_STATUS ArcSetSataHddSpeed
	(
		IN BYTE nVendor,
		IN BYTE nMode		
	);		

	/************************************************************************************
	Function Name:	ArcAtaPassthroughCmd
	Function:		Execute ATA PassThrough Commands
	Input:			1. nEncnum: Enclosure#
					2. nDrvnum: Drive# in Enclsoure#
	                3. Pointer to a sATA_CMD structure containing ATA commands
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _ATA_CMD_MODE
    {
	  PIO_NON_DATA = 0,
	  PIO_DATA_IN,
	  PIO_DATA_OUT,
	  TOTAL_ATA_CMD_MODE
    }ATA_CMD_MODE;

    typedef struct _ATA_PASSTHROUGH_COMMAND
    {
	  struct
	  {
 	    BYTE features;       // [in]   
		BYTE sector_count;	 // [in]   
		BYTE sector_number;	 // [in]
		BYTE cylinder_low;	 // [in/out]
		BYTE cylinder_high;  // [in/out]
		BYTE device_head;    // [in]
		BYTE command;        // [in]
		BYTE error;          // [out]
		BYTE status;         // [out]
		BYTE reserved[6];    //
	  }regs;
	  BYTE command_mode;		 // [in] see ATA_CMD_MODE
	  BYTE data[512];			 // [in/out] buffer for outgoing/incoming data
    }sATA_CMD, *pATA_CMD;

	virtual ARC_STATUS ArcAtaPassthroughCmd
	(
		IN BYTE nEncnum,
		IN BYTE nDrvnum,
		IN sATA_CMD *cmd
	);	



	/************************************************************************************
	Function Name:	ArcScsiPassthroughCmd
	Function:		Execute SCSI PassThrough Commands
	Input:			1. nEncnum: Enclosure#
					2. nDrvnum: Drive# in Enclsoure#
	                3. Pointer to a sSCSI_CMD structure containing SCSI commands
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _SCSI_CMD_MODE
    {
	  SCSI_NON_DATA = 0,
	  SCSI_DATA_IN,
	  SCSI_DATA_OUT,
	  TOTAL_SCSI_CMD_MODE
    }SCSI_CMD_MODE;

	typedef struct _SCSI_PASSTHROUGH_COMMAND
    {
		BYTE cdb[16];            // SCSI CDB
		BYTE cdblen;             // cdb length
		BYTE command_mode;		 // [in] see SCSI_CMD_MODE
		BYTE reserved[2];
		DWORD datalen;           // [in/out] length for outgoing/incoming data, zero for non-data
		BYTE data[512];			 // [in/out] buffer for outgoing/incoming data
    }sSCSI_CMD, *pSCSI_CMD;

	virtual ARC_STATUS ArcScsiPassthroughCmd
	(
		BYTE nEncnum, 
		BYTE nDrvnum, 
		IN sSCSI_CMD *cmd
	);


	/************************************************************************************
	Function Name:	ArcSetWriteSameForInit
	Function:		Set Write Same For Initialization
	Input:			1. nMode: Mode of Write Same For Initialization
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/		
	typedef enum _WRITE_SAME_USAGE
    {
	  WRITE_SAME_SAS_AND_SATA = 0,
	  WRITE_SAME_SAS_ONLY,
	  WRITE_SAME_SATA_ONLY,
	  TOTAL_WRITE_SAME_USAGE
    }WRITE_SAME_USAGE;
	virtual ARC_STATUS ArcSetWriteSameForInit
	(
		IN BYTE nMode		
	);	

	/************************************************************************************
	Function Name:	ArcSetHotPluggedDiskForRebuilding
	Function:		Set Hot Plugged Disk For Rebuilding
	Input:			1. nMode: Mode of Hot Plugged Disk
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/		
	typedef enum _HOTPLUGGED_DISK_FOR_REBUILDING
    {
	  HOTPLUGGED_DISK_BLANK_ONLY = 0,
	  HOTPLUGGED_DISK_ALWAYS,
	  HOTPLUGGED_DISK_DISABLED,
	  TOTAL_HOTPLUGGED_DISK_FOR_REBUILDING
    }HOTPLUGGED_DISK_FOR_REBUILDING;
	virtual ARC_STATUS ArcSetHotPluggedDiskForRebuilding
	(
		IN BYTE nMode		
	);	

	/************************************************************************************
	Function Name:	ArcGenerateAESKeyBuffer
	Function:		Generate AES Key
	Input:			1. nVolNumber: Number of Volumes with encryption capability use to create AES key, 
	                               use all Volumes to create AES key if 0
	                2. nVolArray: Pointer to an array containing index of Volumes with 
					              encryption capability
	Output:			1. nKeyBufSize: [IN]  The buffer size of pKeyBuf
	                                [OUT] The data length in pKeyBuf
	                2. pKeyBuf: Pointer to a buffer containing AES key	
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcGenerateAESKeyBuffer
	(	    		
		IN	   BYTE  nVolNumber,
		IN     BYTE  nVolArray[],
		IN OUT DWORD *nKeyBufSize,		
		OUT    BYTE  *pKeyBuf
	);

	/************************************************************************************
	Function Name:	ArcSetScheduleVolumeCheck
	Function:		Volume check scheduler
	Input:			1. nScheduler: Run consistency checks on a regular basis
	                2. nCheckmode: Checking after system idle for N minute
					3. nScrubBadBlock: Scrub bad block if bad block is found, 0 == enabled, 1 == disabled
					4. nReComputeParity: Re-compute parity if parity error is found, 0 == enabled, 1 == disabled
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/	
	typedef enum _SCHEDULER
	{
		SCHEDULER_DISABLED,
        SCHEDULER_1DAY,
        SCHEDULER_1WEEK,
        SCHEDULER_2WEEKS,
        SCHEDULER_3WEEKS,
        SCHEDULER_4WEEKS,
        SCHEDULER_8WEEKS,
        SCHEDULER_12WEEKS,
        SCHEDULER_16WEEKS,
        SCHEDULER_20WEEKS,
        SCHEDULER_24WEEKS,
		TOTAL_SCHEDULER
	}SCHEDULER;

	typedef enum _CHECKMODE
	{
		CHECKMODE_NO,
        CHECKMODE_1MIN,
        CHECKMODE_3MINS,
        CHECKMODE_5MINS,
        CHECKMODE_10MINS,
        CHECKMODE_15MINS,
        CHECKMODE_20MINS,
        CHECKMODE_30MINS,
        CHECKMODE_45MINS,
        CHECKMODE_60MINS,
		TOTAL_CHECKMODE
	}CHECKMODE;

	virtual ARC_STATUS ArcSetScheduleVolumeCheck
	(
		IN BYTE nScheduler,
		IN BYTE nCheckmode,
		IN BYTE nScrubBadBlock,
		IN BYTE nReComputeParity
	);
	
public:
	virtual ARC_STATUS ArcCmdReadWrite(BYTE cmd, void *w_buf, int w_buf_len, void *r_buf, int r_buf_len, int &actual_read_size, DWORD flag = 0);	
//
// helper functions
//	
	virtual ARC_STATUS ArcHelpBuildDeviceMap(sDRV_MAP *drvMap);		// enumerating disk 
	virtual ARC_STATUS ArcHelpBuildRaidSetMap(sRAID_MAP *raidMap);	// enumerating raidset
	virtual ARC_STATUS ArcHelpBuildVolumeSetMap(sVOL_MAP *volMap);	// enumerating volumeset	

protected:	
	ARC_STATUS ArcGetEvent(char nPage);
	ARC_STATUS SetLogoPage(BYTE nPage, BYTE *Data, int len);
	ARC_STATUS ArcGetSetupPage(BYTE *buf, int bufsize, CFieldInfo *pFIELD_INFO = NULL);
	ARC_STATUS ArcWriteSetupData(int setupitem, char *data, int datalen, CFieldInfo *pFIELD_INFO = NULL);
	ARC_STATUS ArcReadSetupData(int setupitem, char *buf, int buflen, int *retlen, CFieldInfo *pFIELD_INFO = NULL);
	ARC_STATUS ArcGetModelNumber(DWORD *model); 	
	virtual int ArcDoIO(sIF_CMD_BLOCK *icb);
	BOOL	IsValidHandle();
	int DetermineTableNumber();	
	sSYSTEM_INFO m_SysInfo;
	int     m_iIsEnc;

private:
	// save the previous index of event
	int		m_iPreIndex;
	// save the current index of event
	int		m_iCurrentIndex;
	int		m_iNewCount;
	BYTE	*m_pEventPage;
	BYTE	*m_pEventMapping;
	int		m_iMtu;		
	int     m_iTableNum;
};

#pragma pack()

#endif	// # _ARCLIB_H

